package LectureGame.Core;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 10.jan.2007
 * Time: 16:30:44
 * This class runs as a separate thread and listens for StudentClient connections.
 *
 */
class ConnectionListener implements Runnable {

    private final int maxConnections;
    private boolean serverIsAlive = true;
    private final NetworkConnectionKeeper networkConnectionKeeper;

    /**
     * Thread which listens for incoming connections from studentclients
     * @param networkConnectionKeeper
     * @see LectureGame.Core.NetworkConnectionKeeper
     */
    public ConnectionListener(NetworkConnectionKeeper networkConnectionKeeper) {
        this.networkConnectionKeeper = networkConnectionKeeper;
        maxConnections = LG.SERVER_MAX_CONNECTIONS;
    }


    /**
     * Listens for new connections of the type StudentClient. Adds them to the networkConnectionKeeper.
     * It's possible to set a limit to the number of allowed simultanous connections i the LectureGame.Core.LG class
     * @see Runnable
     * @see LectureGame.Core.LG
     */
    public void run() {

        ServerSocket listener = null;
        Socket server;
        try {
            listener = new ServerSocket(LG.SERVER_PORTNUMBER);

        } catch (IOException e) {
            ConsoleWriter.printErr("Could not listen on port: " + LG.SERVER_PORTNUMBER);
            System.exit(-1);
        }

        try {

            // counter that keeps track of connected clients
            int printServerFullSwitch = 0;
            while (serverIsAlive) {
                //debug
                int connectionCounter1 = networkConnectionKeeper.getConnectionCounter();

                if ((connectionCounter1 < maxConnections) || (maxConnections == 0)) {
                    printServerFullSwitch = 0;
                    ConsoleWriter.print("Waiting for connection");

                    //blokker
                    server = listener.accept();

                    //mekanikk for  starte en ny trd
                    StudentClient clientStudentClient = new StudentClient(server, networkConnectionKeeper);

                    //tar vare p den nye connectionen
                    networkConnectionKeeper.addConnection(clientStudentClient);

                    Thread thread = new Thread(clientStudentClient);
                    ConsoleWriter.print("New connection from: " + server.getInetAddress().getHostName() + " (" + server.getInetAddress().getHostAddress() + ")");
                    thread.start();

                    this.serverIsAlive = networkConnectionKeeper.getServerIsAlive();
                } else if (printServerFullSwitch == 0) {
                    ConsoleWriter.printErr("SERVER FULL! " + connectionCounter1 + " connections");
                    printServerFullSwitch = 1;
                } else {
                    Thread.sleep(1000);
                }

            }

        }
        catch (Exception e) {
            ConsoleWriter.printErr("Could not accept client connection");
            e.printStackTrace();
        }

    }

}


