package LectureGame.Core;

import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 22.mar.2007
 * Time: 13:55:49
 * This class must be run as a thread. Watches the connected clients and tests to check for problems. Ensures that
 * clients which are not disconnected gracefully are disconnected and terminated. Current implementation runs this check
 * every 2000 ms.
 */
class ConnectionProber implements Runnable {
    private boolean alive = true;
    private Vector<StudentClient> connections;
    private NetworkConnectionKeeper netkeeper;



    public ConnectionProber() {
        connections = new Vector<StudentClient>();
    }

    
    public void run() {
        while (alive) {
            connections = netkeeper.getConnections();
            MasterClient master = netkeeper.getMaster();
            long currentTime= System.currentTimeMillis();

            for (int i = 0; i < connections.size(); i++) {
                StudentClient c = connections.get(i);
                long delta = currentTime - c.getTimeStampLastReceivedData();

                if (delta > 600000) {
                    if (!c.loginStatus()) {
                        c.sendData("@^DIE0|0^@");
                        c.close();
                        ConsoleWriter.debug("Timout oppdaget i connection fra: " + c.clientIP, 3);
                    }
                }

                if (c.testTrouble()) {
                    c.stop();
                    c.wakeme();
                    System.out.println("connection stoppet!!!");
                }


            }

            if (master != null) {
                
                if (master.testTrouble()) {
                    master.stop();
                    master.wakeme();
                    System.out.println("connection stoppet!!!");
                }

            }
            System.out.println("ANTALL CONNECTIONS:"+connections.size());

            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

        }

    }

    public void setNetworkConnectionKeeper(NetworkConnectionKeeper n) {
        netkeeper = n;
    }

    public void kill() {
        alive = false;
    }
}
