package LectureGame.Core;


import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.SocketException;

/**
 * The MasterClient class inherits from the StudentClient class, and is in that respect a special case of the studen client
 * Differs from the StudentClient class in the way the class handles new conenctions and how it handles replies from the
 * connected peer.
 * @see MasterClient
 * @see NetworkConnectionKeeper
 * @see NetworkManager
 *
 */

public class MasterClient extends StudentClient {
    private ServerSocket serverSocket;
    private NetworkManager manager;

    public MasterClient(NetworkConnectionKeeper n, NetworkManager manager) {
        super(n);
        this.manager = manager;
    }

  private synchronized void init() {

        try {
            server = serverSocket.accept();
            parent.addMaster(this);
            clientName = server.getInetAddress().getHostName();
            clientIP = server.getInetAddress().getHostAddress();

            in = new DataInputStream(server.getInputStream());

            inBufferedReader = new BufferedInputStream(in);
            lineOut = new PrintWriter(server.getOutputStream());
            ConsoleWriter.print("Masterconnection from: " + clientIP + " established");
            //sover for  ta hensyn til network lag
            connectionstate = ConnectionState.READY;
            Thread.sleep(2000);
    }
        catch (SocketException e) {
            //hvis klienten ikke disconnecter skikkelig:
            isConnected = false;
            e.printStackTrace();
            ConsoleWriter.printErr("\nConnection to client: " + clientIP + " (" + clientName + ") lost");


        } catch (Exception e) {
            isConnected = false;
            ConsoleWriter.printErr("Exception in LectureGame.Core.StudentClient");


        }

    }


   private void parse(String received) {
        ConsoleWriter.debug("Parser " + received, 3);
        int numberOfFields = 0;
        String[] stringList = new String[LG.MAX_NUMBER_OF_FIELDS_IN_MSG];

        if (received != null) {

            String code = received.substring(0, 3);
            received = received.substring(3, received.length());
            while (numberOfFields < LG.MAX_NUMBER_OF_FIELDS_IN_MSG) {
                int msgLength = received.indexOf("|");
                if (msgLength == (-1)) {
                    stringList[numberOfFields] = received.substring(0, received.length());
                    break;
                } else {
                    stringList[numberOfFields] = received.substring(0, Math.abs(msgLength));
                    received = received.substring(msgLength + 1, received.length());
                    numberOfFields++;
                }
            }
            // *LOGIN*
            if (code.equals("LGN")) {

                int lgnCode = db.getUser(stringList[0], stringList[1]);

                if (lgnCode == LG.ERROR_LOGIN_OK) {
                    loggedIn = true;
                    parent.addMaster(this);
                    username = stringList[0];
                    password = stringList[1];
                    ConsoleWriter.print("Welcome MASTER " + username + ". Your Password is: " + password);
                    String sender = db.getAllLectures();
                    ConsoleWriter.debug("Master", "Sender: " + sender, 2);
                    lineOut.println("@^LGN" + sender + "^@");
                    lineOut.flush();

                } else if (lgnCode == LG.ERROR_USER_EXIST_WRONG_PW) {
                    lineOut.print("@^LGNNO^@");
                    lineOut.flush();
                } else {
                    lineOut.print("@^LGNUNKNOWN^@");
                    lineOut.flush();
                }
            }

            //svar fra masteren med koden som skal brukes videre
            else if (code.equals("LTR")) {

                lectureCode = stringList[0];
                ConsoleWriter.print("Master", "Lecture code: " + lectureCode);
                manager.setLecture(Integer.parseInt(lectureCode));

            } else if (code.equals("QST")) {
                manager.setState(LG.STATE_PUSH_QUESTION);
            }
            // *REGISTER NEW USER*
            else if (code.equals("REG")) {
                  // Enter code for handeling new MasterClients here. Not currently implemented
            } else if (code.equals("DIE")) {
                close();
            }

            //ping greier.
            else if (code.equals("PNG")) {
                if (pingCounter < pinglist.length) {

                    endPing = System.currentTimeMillis();
                    if (pingCounter > 3) pingSleep += 250;

                    if (pingCounter == 0) pinglistClient[pingCounter][0] = Integer.parseInt(stringList[0]);
                    else
                        pinglistClient[pingCounter][0] = Integer.parseInt(stringList[0]) - pinglistClient[pingCounter - 1][1];

                    pinglistClient[pingCounter][1] = (int) pingSleep;

                    pinglist[pingCounter][0] = endPing - startPing;
                    pinglist[pingCounter][1] = pingSleep;

                    if (pingSleep != 0) {
                        try {
                            Thread.sleep(pingSleep);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }

                    startPing = System.currentTimeMillis();
                    sendData("@^PNG^@");

                    ConsoleWriter.printErr("PING ITERASJON NR " + pingCounter);
                    pingCounter++;

                } else {

                    ConsoleWriter.printErr("***************************************");
                    ConsoleWriter.printErr("PING RESULTERTE I FLGENDE " + pinglist.length + " INTERVALLER:");
                    for (int i = 0; i < pinglist.length; i++) {
                        ConsoleWriter.printErr("SERVER INTERVALL " + (i + 1) + ": " + (pinglist[i][0]) + " ms  |  Delay mellom ping: " + pinglist[i][1]);

                    }
                    ConsoleWriter.printErr("***************************************");
                    for (int i = 0; i < pinglist.length; i++) {
                        ConsoleWriter.printErr("KLIENT INTERVALL " + (i) + ": " + (pinglistClient[i][0]) + " ms  |  Delay mellom ping: " + pinglistClient[i][1]);
                    }
                    ConsoleWriter.printErr("***************************************");
                }
            }
        }


    }


    public void run() {
        init();

        try {
            ConsoleWriter.debug("Master", "connected", 3);

            while (isConnected) {

                ConsoleWriter.debug("Master", "Thread awake", 4);

                // tester om det er trbbel med linja
                if (testTrouble()) {
                    isConnected = false;
                    break;
                } else {
                    parse(receive());

                    //Trden venter til wakeme() blir kjrt av NetWorkManageren
                    ConsoleWriter.debug("Master", "Thread sleeps", 4);
                    connectionWait();
                }
            }

            close();

        } catch (Exception e) {
            ConsoleWriter.printErr("Master", "feil i lesing av datastrom fra masterKlienten" + clientIP);
            close();
            e.printStackTrace();
        }

    }

    /**
     * Closes the connection and all streams which are open
     */
    void close() {

        try {
            isConnected = false;
            if (in != null) in.close();
            if (inBufferedReader != null) inBufferedReader.close();
            if (lineOut != null) lineOut.close();
            if (this.server != null) this.server.close();
            parent.removeMaster();
            ConsoleWriter.printErr(LG.SERVER_NAME + ": MASTER " + clientIP + " has been disconnected");

        }
        catch (Exception e) {
            ConsoleWriter.printErr("LectureGame.Core.StudentClient: connection closed with errors");
            e.printStackTrace();
        }

        manager.listenForMasterClient();
    }


    /**
     * Sets the serversocket to be used for listening for new incoming connections
     * @param socketListener a ServerSocket
     * @see java.net.ServerSocket
     */
    public void setServerSocket(ServerSocket socketListener) {
        serverSocket = socketListener;
    }
}