package LectureGame.Core;

import LectureGame.DataBase.DataBaseConnection;
import LectureGame.GameMode.EliminationMode;
import LectureGame.GameMode.GameMode;
import LectureGame.GameMode.MeasureUpMode;

import java.io.IOException;
import java.net.ServerSocket;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 26.jan.2007
 * Time: 13:05:41
 * The nerver center of the server. This class handles the responses from both StudentClients and the Masterclient.
 * Uses the DataBase LectureGame.DataBase package for database connectivity, and GameMode interface for selecting GameMode.
 *
 * @see GameMode
 * @see StudentClient
 * @see MasterClient
 * @see NetworkConnectionKeeper
 */
public class NetworkManager implements Runnable {

    private NetworkConnectionKeeper connectionKeeper;
    private int serverState = LG.STATE_NO_CONNECTIONS;
    private Question currentQuestion;
    private int[] roundList;
    private DataBaseConnection db;
    private int roundIndex;
    private int totalNumberOfQuestions;
    private int totalPossibleScore;
    private GameMode gameMode;
    private Vector<StudentClient> connections;
    private MasterClient master;
    private ServerSocket socketListener;
    private boolean serverIsAlive;

    /**
     * Constructor of the NetworkManager class.
     */

    public NetworkManager() {
        serverIsAlive = true;
        connectionKeeper = new NetworkConnectionKeeper(this);
        db = connectionKeeper.getDatabase();
        ConnectionListener connectionListener = new ConnectionListener(connectionKeeper);
        Thread thread = new Thread(connectionListener);
        thread.start();
        connections = connectionKeeper.getConnections();
        master = connectionKeeper.getMaster();
        ConnectionProber connectionProber = new ConnectionProber();
        connectionProber.setNetworkConnectionKeeper(connectionKeeper);
        new Thread(connectionProber).start();
        try {
            socketListener = new ServerSocket(LG.SERVER_MASTER_CLIENT_PORTNUMBER);
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Starts a new MasterClient thread. Instansiated when a masterconnection is disconnected or when NetworkManager
     * is instanciated
     *
     * @see LectureGame.Core.MasterClient
     */
    public void listenForMasterClient() {
        MasterClient master = new MasterClient(connectionKeeper, this);
        master.setServerSocket(socketListener);
        Thread thread = new Thread(master);
        thread.start();
    }


    /**
     * Contains main loop which queries the connections for input data according to the server state.
     */
    public void run() {

        listenForMasterClient();

        while (serverIsAlive) {
            try {
                action();

                Thread.sleep(10);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * This method performs actions on the connected clients and on the GameMode according to the state its in.
     * possible states are:
     * <p/>
     * - NO CONNECTIONS : no clients connected
     * - CONNECTED : clients connected
     * - PUSH QUESTIONS : question is to be pushed to the clients
     * - WAITING REPLY : waiting for question answers from the clients
     * - END OF ROUND : end of the round
     * - END OF LECTURE : end of the lecture
     * - LOAD NEW LECTURE : loading a new lecture
     * - PING : ping, handles a ping session
     */

    private void action() {

        master = connectionKeeper.getMaster();

        if ((master != null || connections.size() != 0)) {
            //if(true){

            // ****** NO CONNECTIONS ********
            String res1;
            if (serverState == LG.STATE_NO_CONNECTIONS) {
                serverState = LG.STATE_CONNECTED;
                ConsoleWriter.debug("Setting serverstate " + serverState, 3);
            }

            // ****** CONNECTED ********
            else if (serverState == LG.STATE_CONNECTED) {

                if (master != null) {
                    if (master.inDataAvailiable()) {
                        master.wakeme();
                    }
                }

                for (int i = 0; i < connections.size(); i++) {
                    StudentClient c = connections.get(i);
                    if (c != null && c.inDataAvailiable()) {
                        c.wakeme();
                    }
                }
            }

            // ****** PUSH QUESTIONS ********
            else if (serverState == LG.STATE_PUSH_QUESTION) {


                currentQuestion = gameMode.nextQuestion();
                if (currentQuestion == null) {
                    serverState = LG.STATE_END_OF_ROUND;
                    ConsoleWriter.debug("Setting serverstate " + serverState, 3);
                    return;
                }
                totalNumberOfQuestions++;
                ConsoleWriter.debug("inne i STATE_PUSH_QUESTION", 3);

                // sjekker at vi har et sprsml.

                // henter ut sprsmlet som skal sendes til klientene, og lager
                // samtidig sprsmlet som skal sendes til masteren. forskjellen ligger i at riktig
                // alternativ sendes med spm til master.
                if (gameMode.getQuestionState() == LG.QUESTIONSTATE_FINAL_IN_ROUND && currentRoundIsLastRound()) {
                    gameMode.setQuestionState(LG.QUESTIONSTATE_FINAL_IN_LECTURE);
                }

                String masterQst = gameMode.createMasterQuestion();

                if (master != null) master.sendData(masterQst);

                for (int i = 0; i < connections.size(); i++)
                    if (connections.get(i).loginStatus()) connections.get(i).sendData("A");

                try {
                    //her settes delayet som skal fre til at mobilene er mest mulig synchet.
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }


                String clientQuestion = gameMode.createClientQuestion();

                for (int i = 0; i < connections.size(); i++)
                    if (connections.get(i).getAllowedToAnswer()) connections.get(i).sendQuestion(clientQuestion);

                serverState = LG.STATE_WAITING_REPLY;
                ConsoleWriter.debug("Setting serverstate " + serverState, 3);

            }

            // ****** WAITING REPLY ********
            else if (serverState == LG.STATE_WAITING_REPLY) {
                //setter maks totalScore mulig  oppn
                totalPossibleScore += gameMode.getMaxScore();

                try {
                    Thread.sleep(currentQuestion.getTimeLimit() * 1000 + 1500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

                for (int i = 0; i < connections.size(); i++) {
                    StudentClient c = connections.get(i);
                    if (c.inDataAvailiable()) {
                        // gir tillatelse til at connectionen kan svare;
                        c.wakeme();
                    }
                }
                try {
                    Thread.sleep(300);
                } catch (InterruptedException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                res1 = gameMode.handleQuestionReply();
                if (master != null) master.sendData(res1);
                int gameModeQuestionState = gameMode.getQuestionState();

                if (gameModeQuestionState == LG.QUESTIONSTATE_NOT_FINAL) serverState = LG.STATE_CONNECTED;
                else serverState = LG.STATE_END_OF_ROUND;

                ConsoleWriter.debug("Setting serverstate " + serverState, 3);
            }

            // *** END OF ROUND ***
            else if (serverState == LG.STATE_END_OF_ROUND) {

                res1 = gameMode.handleEndOfRound();
                if (master != null) master.sendData(res1);
                nextRound();

            }

            // *** END OF LECTURE ***
            else if (serverState == LG.STATE_END_OF_LECTURE) {
                String res = handleEndOfRound();
                if (master != null) master.sendData(res);
                serverState = LG.STATE_CONNECTED;
                makeClientsDie();
                ConsoleWriter.debug("Setting serverstate " + serverState, 3);

            }

            // *** LOAD NEW LECTURE ***
            else if (serverState == LG.STATE_NEW_LECTURE) {
                resetStatistics();
                nextRound();
                serverState = LG.STATE_CONNECTED;
                ConsoleWriter.debug("Setting serverstate " + serverState, 3);


            }

            // ****** PING ********
            else if (serverState == LG.STATE_PING) {

                for (StudentClient c : connections) c.triggerPing();
                serverState = LG.STATE_CONNECTED;
                ConsoleWriter.debug("Setting serverstate " + serverState, 3);

            }

        } else if (connections.isEmpty() && master == null && serverState != LG.STATE_NO_CONNECTIONS) {
            ConsoleWriter.debug("Waiting for Connections", 3);
            serverState = LG.STATE_NO_CONNECTIONS;
            ConsoleWriter.debug("Setting serverstate " + serverState, 3);
        }
    }

    private void resetStatistics() {
        totalPossibleScore = 0;
        totalNumberOfQuestions = 0;
        roundIndex = 0;

    }

    private void makeClientsDie() {
        int counter = connections.size();

        for (int i = 0; i < counter; i++) {
            StudentClient c = connections.get(i);
            c.sendData("@^DIE" + c.getTotalScore() + "|" + totalPossibleScore + "^@");
        }

    }


    /**
     * Sets the server state. See LectureGame.Core.LG for the list of possible constants
     *
     * @param state
     * @see LG
     */
    public void setState(int state) {
        serverState = state;
        ConsoleWriter.debug("Setting serverstate " + serverState, 3);
    }

    /**
     * Checks the answer using handeling code in the class implementing the GameMode interface
     *
     * @param answer the answer from the client, a string representing an integer
     * @param c      the client which answered
     * @return 1 if correct, 0 if false
     */
    public int checkAns(String answer, StudentClient c) {
        return gameMode.checkAns(answer, c);
    }

    /**
     * Method used for controlling how many questions a round contains
     *
     * @return the total number of questions in a round
     */
    public int getTotalNumberOfQuestions() {
        return totalNumberOfQuestions;
    }

    /**
     * Generates a string to be sent to the clients
     *
     * @param roundScore score acheived in this round
     * @param totalScore total score accumulated
     * @return string formatted for sending
     */
    public String generateANS(int roundScore, int totalScore) {
        String returnvalue = gameMode.handleClientQuestionReply(roundScore, totalScore, totalPossibleScore);
        ConsoleWriter.debug("Sender " + returnvalue, 3);
        return returnvalue;
    }


    /**
     * This is where the GameMode is set according to the GameMode code registered in the database in the round table.
     * Any additional game modes must be added here in a new 'else if'. Also the new GameMode code must be added to the
     * LG constants class.
     *
     * @param gamemode
     */
    private void setGameMode(int gamemode) throws IllegalGameModeException {

        if (gamemode == LG.GAMEMODE_BEAT_THE_SCORE) {
            // fokuser p denne
            gameMode = new MeasureUpMode(connectionKeeper);


        } else if (gamemode == LG.GAMEMODE_LAST_MAN_STANDING) {
            gameMode = new EliminationMode(connectionKeeper);

        } else {
            throw new IllegalGameModeException("No such game mode");
        }
        master.sendData("@^MOD" + gamemode + "^@");

    }

    /**
     * Returns which round is the current round
     *
     * @return current round
     */

    public int getCurrentRound() {
        if (roundIndex < roundList.length) return roundList[roundIndex];
        return -1;
    }

    /**
     * Sets tje lecture id where the rounds are to be extracted from
     *
     * @param lectureID the lectureID to be used
     */
    public void setLecture(int lectureID) {
        roundList = db.getRoundsInLecture(lectureID);
        roundIndex = 0;
        totalNumberOfQuestions = 0;
        setState(LG.STATE_NEW_LECTURE);
    }

    /**
     * Stops the networkmanager and closes all connections.
     */
    public void stop() {
        serverIsAlive = false;
        for (int i = 0; i < connections.size(); i++) connections.get(i).close();
        connectionKeeper.getMaster().close();
    }


    /**
     * Sets the next round and loads the questions in it. or if the current round is the final one in the lecture then the serverstate
     * END OF LECTURE is set.
     *
     * @return true if the method executed correctly, false if not
     */
    private boolean nextRound() {

        if (roundIndex == roundList.length) {
            serverState = LG.STATE_END_OF_LECTURE;
            ConsoleWriter.printErr("-- END OF LECTURE --");
        } else if (roundIndex < roundList.length) {
            try {
                setGameMode(db.getGamgeMode(roundList[roundIndex]));
                gameMode.initiateRound();
                serverState = LG.STATE_CONNECTED;
                roundIndex++;
                return true;
            } catch (IllegalGameModeException e) {
                e.printStackTrace();
            }
        }
        return false;
    }


    /**
     * Handles the end of a round, building the end of round string to be sendt to the MasterClient
     *
     * @return string to be sendt to MasterClient
     */
    private String handleEndOfRound() {

        StringBuilder s = new StringBuilder("@^LCT");
        int clientSize = connections.size();
        s.append(clientSize);
        for (int i = 0; i < clientSize; i++) {
            s.append("|");
            s.append(connections.get(i).getUsername());
            s.append("|");
            s.append(connections.get(i).getTotalScore());

        }

        s.append("^@");
        return s.toString();

    }

    /**
     * Performs check to see if the current round is the last round in a lecture
     *
     * @return is the round the last round
     */
    private boolean currentRoundIsLastRound() {
        return roundIndex == roundList.length;
    }


}
