package LectureGame.Core;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 26.jan.2007
 * Time: 16:40:05
 * This class is a generalization of a question, as used in the LectureGame.Core.LG
 */
public class Question {

    private String question;
    private int numberOfAlt;
    private String[] alternatives;
    private String timeLimit;
    private String correctAlternative;
    private String formattedQuestion;

    /**
     * @param question Question text. example "who am i?"
     * @param numberOfAlt number of answers to choose from
     * @param alternatives Array of alternatives. example {"me","you","them"}
     * @param timeLimit time given to reply to the question. 0 = the server decides
     * @param correctAlternative index number of the alternative in the Array of alternatives that is correct. Can only be one
     */
    public Question(String question, int numberOfAlt, String[] alternatives, String timeLimit, String correctAlternative) {

        this.question = question;
        this.numberOfAlt = numberOfAlt;
        this.alternatives = alternatives;
        this.timeLimit = timeLimit;
        this.correctAlternative = correctAlternative;
        this.formattedQuestion = null;
    }

    /**
     * Creates the root of the question to be sendt to eighter the studenclients or the masterclient. The
     * root is similar, thus this only needs one method. The last part of the question is different though.
     *
     * Example of output:
     *
     * @^QSTWho am I?|3|me|you|them|30|
     *
     * this is translated to
     * question: who am I?
     * number of alternatives: 3
     * alternative 1: me
     * alternative 2: you
     * alternative 3: them
     * timelimit: 30 s
     *
     * @return  a question root
     */
    private String makeQuestionRoot() {
        if (formattedQuestion != null) return formattedQuestion;
        formattedQuestion = "@^QST" + question + "|" + Integer.toString(numberOfAlt);
        for (String alternative : alternatives) formattedQuestion += "|" + alternative;
        return formattedQuestion += "|" + timeLimit +"|";
    }


    /**
     * This method is used for testing purposes. prints the question to console.
     */
    public void testQuestion() {
        System.out.println("sprsml: " + question);
        for (int i = 0; i < alternatives.length; i++) System.out.println("alt #" + i + ": " + alternatives[i]);
        System.out.println("   ***   ");
    }

    /**
     * Returns the correct alternative, an index of the String array as entered when creating the question
     * @return the correct alternative (formatted as String since it is to be sendt to the clients)
     */
    public String getCorrectAnswer() {
        return correctAlternative;
    }


    /**
     * Returns the timelimit of the question
     * @return  timelimit
     */
    public int getTimeLimit() {
        return Integer.parseInt(timeLimit);
    }

    // forutsetter at det siste parameteret er om sprsmlet er det siste som sendes i runden eller ikke
    // forutsetning n: 0 = ikkesiste, 1 = siste i runde, 2 = siste i lecture;
    // det under her blir feil.. correctAnswer blir nest siste parameter som sendes til masteren.

    /**
     * Creates the question to be sendt to the StudentClient. Appends:
     * 0 - not last question in lecture
     * 1 - last question in round
     * 2 - last question i lecture
     *
     * @param questionState the state of the question
     * @return String formatted and ready to be sendt to the StudentClient
     */
    public String makeClientQuestion(int questionState){

        String qstString = makeQuestionRoot();
        StringBuilder masterQst = new StringBuilder(qstString);

        if (questionState == LG.QUESTIONSTATE_NOT_FINAL) {
             masterQst.append(LG.QUESTIONSTATE_NOT_FINAL);
            masterQst.append("^@");
            return masterQst.toString();
        } else if (questionState == LG.QUESTIONSTATE_FINAL_IN_ROUND) {
            masterQst.append(LG.QUESTIONSTATE_FINAL_IN_ROUND);
            masterQst.append("^@");
            return masterQst.toString();
        } else if (questionState == LG.QUESTIONSTATE_FINAL_IN_LECTURE) {
            masterQst.append(LG.QUESTIONSTATE_FINAL_IN_LECTURE);
            masterQst.append("^@");
            return masterQst.toString();
        }
        return "";
    }

    /**
     * Creates the question to be sendt to the MasterClient. Appends first the correct alternative (which is not sendt
     * to the clients) and then appends:
     *
     * 0 - not last question in lecture
     * 1 - last question in round
     * 2 - last question i lecture
     *
     * @param questionState the state of the question
     * @return String formatted and ready to be sendt to the StudentClient
     * 
     */
    public String makeMasterQuestion(int questionState) {

        String qstString = makeQuestionRoot();
        StringBuilder masterQst = new StringBuilder(qstString);

            masterQst.append(getCorrectAnswer());
            masterQst.append("|");

        if (questionState == LG.QUESTIONSTATE_NOT_FINAL) {
             masterQst.append(LG.QUESTIONSTATE_NOT_FINAL);
            masterQst.append("^@");
            return masterQst.toString();
        } else if (questionState == LG.QUESTIONSTATE_FINAL_IN_ROUND) {
            masterQst.append(LG.QUESTIONSTATE_FINAL_IN_ROUND);
            masterQst.append("^@");
            return masterQst.toString();
        } else if (questionState == LG.QUESTIONSTATE_FINAL_IN_LECTURE) {
            masterQst.append(LG.QUESTIONSTATE_FINAL_IN_LECTURE);
            masterQst.append("^@");
            return masterQst.toString();
        }
        return "";
    }

}