package LectureGame.Core;

import LectureGame.DataBase.DataBaseConnection;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 09.jan.2007
 * Time: 11:59:42
 * To change this template use File | Settings | File Templates.
 */

/*

* f p plass boolske verdier for kontroll av trdene
* */

public class StudentClient implements Runnable {

    protected enum ConnectionState {
        READY,
        NOT_READY
    }

    // package access
    ConnectionState connectionstate;
    String username;
    String password;
    String lectureCode;
    DataBaseConnection db;
    Socket server;
    String clientName;
    String clientIP;
    DataInputStream in;
    BufferedInputStream inBufferedReader;
    PrintWriter lineOut;
    NetworkConnectionKeeper parent;
    boolean isConnected;
    int pingCounter;
    long[][] pinglist;
    int[][] pinglistClient;
    long startPing;
    long endPing;
    long pingSleep;

    // private access
    private int protocolState;
    private long timeoutTimer;
    private boolean isReading;
    private NetworkManager netMan;
    private int totalScore;
    private int roundScore;
    private String answer;
    private boolean isAllowedToAnswer;
    boolean loggedIn;
    private boolean answerDirty;

    // Constructor used by masterclient
    StudentClient(NetworkConnectionKeeper parent) {
        connectionstate = ConnectionState.NOT_READY;
        this.netMan = parent.getNetworkManager();
        isConnected = true;
        isReading = false;
        protocolState = LG.PROT_ENDED_STEP2;
        timeoutTimer = System.currentTimeMillis();
        this.parent = parent;
        username = "notLoggedIn";
        password = "";
        lectureCode = "";
        answer = "";

        db = parent.getDatabase();
        isAllowedToAnswer = false;
        loggedIn = false;
        answerDirty = false;
    }


    // Normal StudentClient Constructor
    StudentClient(Socket server, NetworkConnectionKeeper parent) {
        connectionstate = ConnectionState.NOT_READY;
        this.netMan = parent.getNetworkManager();
        isConnected = true;
        isReading = false;
        protocolState = LG.PROT_ENDED_STEP2;
        timeoutTimer = System.currentTimeMillis();
        this.parent = parent;
        this.server = server;
        username = "notLoggedIn";
        password = "";
        lectureCode = "";
        answer = "";
        db = parent.getDatabase();
        isAllowedToAnswer = false;
        answerDirty = false;
        loggedIn = false;
    }

    // metode for initialisering av input og output
    private void init() {

        try {
            totalScore = 0;
            clientName = server.getInetAddress().getHostName();
            clientIP = server.getInetAddress().getHostAddress();
            in = new DataInputStream(server.getInputStream());

            inBufferedReader = new BufferedInputStream(in);

            lineOut = new PrintWriter(server.getOutputStream());

            ConsoleWriter.print(clientIP, "StudentClient established");
            connectionstate = ConnectionState.READY;
            // sleeps to handle network lag
            Thread.sleep(4000);


        }
        catch (SocketException e) {
            // handles clients which does not disconnect properly
            ConsoleWriter.printErr("StudentClient to client: " + clientIP + " (" + clientName + ") lost");
            isConnected = false;


        } catch (Exception e) {
            ConsoleWriter.printErr("Exception in LectureGame.Core.StudentClient");
            isConnected = false;
            e.printStackTrace();

        }

    }

    public void run() {

        try {
            init();

            while (isConnected) {
                ConsoleWriter.debug(clientIP, "Thread awake", 4);

                // checks the connection for truble
                if (testTrouble()) {
                    isConnected = false;
                    break;
                }

                parse(receive());

                // Thread waits until wakeme() is invoked by NetworkManager
                connectionWait();

            }

            close();


        } catch (Exception e) {

            ConsoleWriter.printErr("Error reading datastream from " + clientIP);
            isConnected = false;
            close();
            e.printStackTrace();
        }

    }


    void close() {

        try {
            isConnected = false;
            if (in != null) in.close();
            if (inBufferedReader != null) inBufferedReader.close();
            if (lineOut != null) lineOut.close();
            if (this.server != null) this.server.close();
            loggedIn = false;
            parent.removeConnection(this);
            ConsoleWriter.printErr(LG.SERVER_NAME + ": " + clientIP + " has been disconnected");
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void parse(String received) {

        int numberOfFields = 0;
        String[] stringList = new String[LG.MAX_NUMBER_OF_FIELDS_IN_MSG];

        if (received != null) {

            String code = received.substring(0, 3);
            received = received.substring(3, received.length());
            while (numberOfFields < LG.MAX_NUMBER_OF_FIELDS_IN_MSG) {
                int msgLength = received.indexOf("|");
                if (msgLength == (-1)) {
                    stringList[numberOfFields] = received.substring(0, received.length());
                    numberOfFields++;
                    break;
                } else {
                    stringList[numberOfFields] = received.substring(0, Math.abs(msgLength));
                    received = received.substring(msgLength + 1, received.length());
                    numberOfFields++;
                }
            }
            // *LOGIN*
            if (code.equals("LGN")) {

                int lgnCode = db.getUser(stringList[0], stringList[1]);

                if (lgnCode == LG.ERROR_LOGIN_OK) {
                    loggedIn = true;
                    username = stringList[0];
                    password = stringList[1];
                    lectureCode = stringList[2];

                    ConsoleWriter.debug("(LGN Debug) mottatt Username=" + username + " ; pw=" + password + " ; lecturecode='" + lectureCode + "'", 4);

                    lineOut.println("@^LGNOK^@");
                    lineOut.flush();

                    if (parent.getMaster() != null) {
                        parent.getMaster().sendData("@^USR" + username + "^@");

                    }
                } else if (lgnCode == LG.ERROR_USER_EXIST_WRONG_PW) {
                    lineOut.println("@^LGNNO^@");
                    lineOut.flush();
                } else {
                    lineOut.println("@^LGNUNKNOWN^@");
                    lineOut.flush();
                }
            }

            // *ANSWER*
            else if (code.equals("ANS")) {

                answer = stringList[0];
                answerDirty = true;

                int receivedScore = netMan.checkAns(answer, this);
                totalScore += receivedScore;
                roundScore += receivedScore;
                lineOut.println(netMan.generateANS(roundScore, totalScore));
                lineOut.flush();

            }
            // *REGISTER NEW USER*
            else if (code.equals("REG")) {
                int regCode = db.newUser(stringList[0], stringList[1]);
                if (regCode == LG.ERROR_LOGIN_OK) {
                    loggedIn = true;
                    username = stringList[0];
                    password = stringList[1];
                    lectureCode = stringList[2];
                    lineOut.println("@^LGNOK^@");
                    lineOut.flush();
                    ConsoleWriter.debug("REGISTRERT Username=" + username + " ; pw=" + password + " ; lecturecode='" + lectureCode + "'", 4);
                    if (parent.getMaster() != null) {
                        parent.getMaster().sendData("@^USR" + username + "^@");

                    }
                } else {
                    lineOut.println("@^LGNNO^@");
                    lineOut.flush();
                }


            }
            // * PING (DEPRECATED) *
            else if (code.equals("PNG")) {

                if (pingCounter < pinglist.length) {
                    endPing = System.currentTimeMillis();

//                    if (pingCounter % 2 == 0) {
//                        pingSleep = 0;
//                    } else {
//                        pingSleep = 30000;
//                    }

                    pingSleep = 30000;

                    ConsoleWriter.printErr("Delay mellom ping: " + pingSleep);
                    if (pingCounter == 0) pinglistClient[pingCounter][0] = Integer.parseInt(stringList[0]);
                    else{
                        pinglistClient[pingCounter][0] = Integer.parseInt(stringList[0]) - pinglistClient[pingCounter - 1][1];
                    }

                    pinglistClient[pingCounter][1] = (int) pingSleep;

                    if (pingCounter != 0) pinglist[pingCounter - 1][0] = endPing - startPing;

                    pinglist[pingCounter][1] = pingSleep;

                    if (pingSleep != 0) {
                        try {
                            Thread.sleep(pingSleep);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }

//                    sendData("A");
//
//                    try{
//                        Thread.sleep(2000);
//                    }catch(Exception e){
//                        e.printStackTrace();
//                    }

                    startPing = System.currentTimeMillis();
                    sendData("@^PNG^@");

                    ConsoleWriter.printErr("PING ITERASJON NR " + pingCounter);
                    pingCounter++;


                } else {

                    System.out.println("---------------------------------------------------------------");
                    System.out.println("PING GAVE THE FOLLOWING " + (pinglist.length-1) + " RESULTS:");
                    System.out.println("INTERVAL #, ROUNDTRIP TIME(ms), DELAY M(ms)");
                    for (int i = 0; i < pinglist.length; i++) {
                    System.out.println((i + 1) + "," + (pinglist[i][0]) + "," + pinglist[i][1]);

                    }
                    ConsoleWriter.printErr("---------------------------------------------------------------");
//                    for (int i = 0; i < pinglist.length; i++) {
//                        ConsoleWriter.printErr("KLIENT INTERVALL " + (i) + ": " + (pinglistClient[i][0]) + " ms  |  Delay mellom ping: " + pinglistClient[i][1]);
//                    }
//                    ConsoleWriter.printErr("---------------------------------------------------------------");
                }
            }
        }


    }

    public void sendData(String message) {
        if(connectionstate == ConnectionState.NOT_READY) return;
        lineOut.println(message);
        lineOut.flush();
        ConsoleWriter.debug(": sending '" + message + "' to " + username + " (" + clientIP + ")", 1);
    }

    protected String receive() {
        if(connectionstate == ConnectionState.NOT_READY) return "";
        String rec = "";
        int ch = 0;
        char[] tempChar = new char[1];
        while (ch != -1) {
            try {
                isReading = true;
                ch = inBufferedReader.read();
                isReading = false;
                tempChar[0] = (char) ch;
                String tempString = new String(tempChar);
                if (ch == -1) break;
                rec = rec + "" + new String(tempChar);
                if ((protocolState == LG.PROT_ENDED_STEP2)) {
                    if (tempString.equals("@")) {
                        protocolState = LG.PROT_STARTED_STEP1;
                    } else {
                        rec = "";

                    }
                } else if ((protocolState == LG.PROT_STARTED_STEP1)) {
                    if (tempString.equals("^")) {
                        protocolState = LG.PROT_STARTED_STEP2;
                    } else {
                        rec = "";

                    }
                } else if (protocolState == LG.PROT_STARTED_STEP2) {
                    if (tempString.equals("^")) {
                        protocolState = LG.PROT_ENDED_STEP1;
                    }
                } else if ((protocolState == LG.PROT_ENDED_STEP1)) {
                    if (tempString.equals("@")) {
                        protocolState = LG.PROT_ENDED_STEP2;
                        break;
                    } else {
                        protocolState = LG.PROT_STARTED_STEP2;
                    }
                }


            }
            catch (IOException e) {
                ch = -1;
            }
        }

        ConsoleWriter.debug("Mottatt fra " + username + ": '" + rec + "'", 3);

        if (rec.length() > 3) {
            return rec.substring(2, (rec.length() - 2));
        }
        return null;

    }


    void wakeme() {
        wakemeSynchronized();
    }

    private synchronized void wakemeSynchronized() {
        notifyAll();
    }

    synchronized void connectionWait() {
        try {
            wait();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public boolean inDataAvailiable() {
        if(connectionstate == ConnectionState.NOT_READY) return false;
        try {

            if (!isReading && inBufferedReader != null) {
                if (inBufferedReader.available() > 0) {
                    timeoutTimer = System.currentTimeMillis();
                    return true;
                }
            }
        } catch (Exception e) {
            ConsoleWriter.printErr(LG.SERVER_NAME + ": connection lost, disconnecting");
            this.close();
        }
        return false;
    }

    public void stop(){
        isConnected = false;
    }

    public long getTimeStampLastReceivedData() {
        return timeoutTimer;
    }

    public boolean loginStatus() {
        return loggedIn;
    }

    public void sendQuestion(String question) {

        if(connectionstate == ConnectionState.NOT_READY) return;
        ConsoleWriter.debug("Sending to " + username + question, 3);
        lineOut.println(question);
        lineOut.flush();
    }

    synchronized boolean testTrouble() {
        if(connectionstate == ConnectionState.NOT_READY) return false;
        if (lineOut != null) {
            lineOut.print("0");
            lineOut.flush();
            return lineOut.checkError();
        }
        return true;
    }

    public void triggerPing() {

        isAllowedToAnswer = true;
        int antallIterasjoner = 50;
        pingSleep = 0;

        pinglist = new long[antallIterasjoner][2];

        pinglistClient = new int[antallIterasjoner][2];
        pingCounter = 0;
        startPing = System.currentTimeMillis();
        endPing = 0;

        sendData("@^PNG^@");

    }


    public String getUsername() {
        return username;
    }

    public String getAnswer() {
        //for  kunne sttte timeout hos klientene m denne mekanikken vre p plass
        //hver gang et svar mottas settes answerdirty til true. settes til false nr lest.
        if (answerDirty) {
            answerDirty = false;
            return answer;
        } else return null;
    }

    public String getTotalScore() {
        return Integer.toString(totalScore);
    }

    public void setAllowedToAnswer(boolean allowedToAnswer) {
        isAllowedToAnswer = allowedToAnswer;
    }

    public boolean getAllowedToAnswer() {
        return isAllowedToAnswer;
    }

}
