package LectureGame.GameMode;

import LectureGame.Core.*;
import LectureGame.DataBase.DataBaseConnection;

import java.util.ArrayList;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 22.feb.2007
 * Time: 16:25:20
 * The elimination mode is a game mode where the partisipants all compete against eachother. As soon as
 * a player answers a question wrong, shes out ouf the round. The round continiues until
 *
 * a) one player is left
 * b) all players are dead
 * c) all questions are depleted
 *
 * Clients logged in after the round has started (i.e. the first question has been posed) cannot partisipate until the
 * round is over (this is by design). It is therefor not recommended to have this gamemode in the very first round.
 *
 * For method descriptions, see the GameMode interface
 *
 * @see LectureGame.GameMode.GameMode
 *
 */
public class EliminationMode implements GameMode {

    private NetworkManager manager;
    private Question currentQuestion;
    private Question[] questionList;
    private DataBaseConnection db;
    private int questionIndex;
    private int roundID;
    private int questionState;
    private int maxScorePerQuestion;
    private NetworkConnectionKeeper netKeeper;
    private ArrayList<StudentClient> survivors;
    private ArrayList<StudentClient> roundPartisipants;
    private int partisipantsLeft;


    public EliminationMode(NetworkConnectionKeeper netKeeper) {
        this.netKeeper = netKeeper;
        maxScorePerQuestion = 1;
        questionState = LG.QUESTIONSTATE_NOT_FINAL;
        manager = netKeeper.getNetworkManager();
        roundID = manager.getCurrentRound();
        db = netKeeper.getDatabase();
        partisipantsLeft = 0;

    }


    public Question nextQuestion() {

        if (questionIndex == 0) {
            survivors = new ArrayList<StudentClient>();
            roundPartisipants = new ArrayList<StudentClient>();

            Vector<StudentClient> c = netKeeper.getConnections();
            int size = c.size();
            for (int i = 0; i < size; i++) {
                StudentClient con = c.get(i);
                if (con.loginStatus()) {
                    survivors.add(con);
                    roundPartisipants.add(con);
                }

            }

            ConsoleWriter.debug("Antall deltakere i LMS:" + survivors.size(),3);
            size = survivors.size();
            for (int i = 0; i < size; i++) {
                StudentClient con = survivors.get(i);
                if (con.loginStatus()) {
                    con.setAllowedToAnswer(true);
                    partisipantsLeft++;
                } else con.setAllowedToAnswer(false);

            }

            currentQuestion = questionList[questionIndex];
            questionIndex++;
            questionState = LG.QUESTIONSTATE_NOT_FINAL;

            if (partisipantsLeft <= 1) {
                questionState = LG.QUESTIONSTATE_FINAL_IN_ROUND;
            }
            return currentQuestion;

        } else if (questionIndex < questionList.length - 1) {

            questionState = LG.QUESTIONSTATE_NOT_FINAL;
            currentQuestion = questionList[questionIndex];
            questionIndex++;
            return currentQuestion;
        } else if (questionIndex == questionList.length - 1) {
            currentQuestion = questionList[questionIndex];
            questionState = LG.QUESTIONSTATE_FINAL_IN_ROUND;
            questionIndex++;
            return currentQuestion;
        }

        manager.setState(LG.STATE_END_OF_ROUND);

        return null;

    }

    public String createMasterQuestion() {
        return currentQuestion.makeMasterQuestion(questionState);
    }

    public String createClientQuestion() {
        return currentQuestion.makeClientQuestion(questionState);
    }

    public void initiateRound() {
        questionList = db.getQuestion(roundID);
    }

    public int getMaxScore() {
        return maxScorePerQuestion;
    }

    public String handleClientQuestionReply(int score, int roundScore, int totalPossibleScore) {
        StringBuilder s = new StringBuilder("@^RES");
        s.append(currentQuestion.getCorrectAnswer());
        s.append("|");
        s.append(score);
        s.append("|");
        s.append(totalPossibleScore);
        s.append("^@");
        return s.toString();

    }

    public int checkAns(String answer, StudentClient c) {
        String correctAns = currentQuestion.getCorrectAnswer();

        if (correctAns.equals(answer)) {
            ConsoleWriter.debug("KORREKT svar fra " + c.getUsername(), 3);
            return 1;
        }
        c.setAllowedToAnswer(false);
        ConsoleWriter.debug("FEIL svar fra " + c.getUsername(), 3);

        return 0;
    }

    public int getQuestionState() {
        return questionState;
    }

    public void setQuestionState(int s) {
        questionState = s;
    }


    public String handleQuestionReply() {

        StringBuilder s = new StringBuilder("@^ANS");
        if (survivors != null) {
            //int clientSize = survivors.size();
            s.append(survivors.size());


            for (int i = 0; i < survivors.size(); i++) {
                StudentClient c = survivors.get(i);
                s.append("|");
                s.append(c.getUsername());
                s.append("|");

                String answer = "0";
                if (c.getAnswer() != null) {
                    if (c.getAllowedToAnswer()) answer = "1";
                } else {
                    ConsoleWriter.debug(c.getUsername() + " TIMOUT!!", 3);
                    c.setAllowedToAnswer(false);
                }
                s.append(answer);

            }
            s.append("^@");


            for (int i = 0; i < survivors.size(); i++) {
                if (!survivors.get(i).getAllowedToAnswer()){
                    survivors.remove(i);
                    i--;
                }
            }

            if (survivors.size() <= 1) questionState = LG.QUESTIONSTATE_FINAL_IN_ROUND;

            return s.toString();

        } else {
            ConsoleWriter.printErr("ERROR: survivors = null");
            return null;
        }

    }

    public String handleEndOfRound() {


        StringBuilder s = new StringBuilder("@^RND");
        int clientSize = roundPartisipants.size();
        s.append(clientSize);

        for (int i = 0; i < clientSize; i++) {
            s.append("|");
            s.append(roundPartisipants.get(i).getUsername());
            s.append("|");
            s.append(roundPartisipants.get(i).getTotalScore());

        }
        s.append("^@");
        return s.toString();

    }
}
