package LectureGame.GameMode;

import LectureGame.Core.Question;
import LectureGame.Core.StudentClient;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 22.feb.2007
 * Time: 13:19:13
 * Interface used to dynamically swap the gameMode. All GameModes must implement this interface.
 */
public interface GameMode {

    /**
     * Method for creating the reply string which is sendt to the masterclient.
     * This string contains the score of all active student clients, and is sendt from the server after each
     * question has timed out.
     *
     * @return a string formatted for sending to the master client, containing the score of all the active student clients
     */
    String handleQuestionReply();

    /**
     * Formats an end of round string for sending to the masterclient
     *
     * @return a formatted string for sending to the master client
     */
    String handleEndOfRound();

    /**
     * Iterates to the the next question in the round. Sets the questionstate according to whether or not the question
     * is the last one in the lecture.
     *
     * @return the next question in the round
     * @see LectureGame.Core.Question
     */
    Question nextQuestion();

    /**
     * Formats a question for sending to the master client.
     *
     * @return a string formatted and ready for sending to the master client
     */
    String createMasterQuestion();

    /**
     * Returns an integer referencing the question state in the LG constants class
     *
     * @return the question state
     * @see LectureGame.Core.LG
     */
    int getQuestionState();

    /**
     * Sets the state of the current question to the state given as an argument.
     * @param s question state (see LG constants class)
     * @see LectureGame.Core.LG
     *
     */
    void setQuestionState(int s);

    /**
     * Uses the current question in the round and formats it for sending to the student clients
     * @return correctly formatted string containing the question
     * @see LectureGame.Core.StudentClient
     * @see Question
     */
    String createClientQuestion();

    /**
     * Extracts all the questions from the current round from the database
     * @see LectureGame.DataBase.DataBaseConnection
     */
    void initiateRound();


    /**
     * Formats the results of the question answers from the studentclients
     * @param score the score received from answering the last question
     * @param roundScore total score in the current round
     * @param totalPossibleScore overall score
     * @return a sting correctly formatted for sending to the student clients
     */
    String handleClientQuestionReply(int score, int roundScore, int totalPossibleScore);

    /**
     * Returns the maximum possible score per question
     * @return max score per question
     * @see Question
     */
    int getMaxScore();


    /**
     * Checks the answer from the studentclients and returns an int depending on what is expected from the gamemode.
     * Typically this method will return 1 for correct and 0 for false.
     * @param answer the answer index represented by a string
     * @param c the client which replied
     * @return an integer (typically 1 for correct, and 0 for false)
     */
    int checkAns(String answer, StudentClient c);

}
