package LectureGame.GameMode;

import LectureGame.Core.*;
import LectureGame.DataBase.DataBaseConnection;

import java.util.Vector;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 22.feb.2007
 * Time: 16:25:20
 * This game mode is more a traditional "answer correctly and receive points" type of quiz. Partisipants logging in
 * after the round has started can also partisipate.
 *
 * For method descriptions, see the GameMode interface
 *
 * @see LectureGame.GameMode.GameMode
 */
public class MeasureUpMode implements GameMode {

    private NetworkManager manager;
    private Question currentQuestion;
    private Question[] questionList;
    private DataBaseConnection db;
    private int questionIndex;
    private int roundID;
    private int questionState;
    private int maxScorePerQuestion;
    private ArrayList<StudentClient> connections;
    private  NetworkConnectionKeeper netKeeper;


    public MeasureUpMode(NetworkConnectionKeeper netKeeper) {
        this.netKeeper = netKeeper;
        maxScorePerQuestion = 1;
        questionState = LG.QUESTIONSTATE_NOT_FINAL;
        manager = netKeeper.getNetworkManager();
        roundID = manager.getCurrentRound();
        db = netKeeper.getDatabase();
        connections = new ArrayList<StudentClient>(netKeeper.getLoggedInConnections());
    }


    public Question nextQuestion() {

        connections = new ArrayList<StudentClient>(netKeeper.getLoggedInConnections());

        if (questionIndex == 0) {
            int size = connections.size();
            for (int i = 0; i < size; i++) {
            StudentClient con = connections.get(i);
                    con.setAllowedToAnswer(true);
            }

            currentQuestion = questionList[questionIndex];
            if (questionIndex == questionList.length - 1)questionState = LG.QUESTIONSTATE_FINAL_IN_ROUND;
            questionIndex++;
            return currentQuestion;

        } else if (questionIndex < questionList.length - 1) {

            int size = connections.size();
            for (int i = 0; i < size; i++) {
                StudentClient con = connections.get(i);
                if (con.loginStatus()) {
                    con.setAllowedToAnswer(true);
                }
            }

            currentQuestion = questionList[questionIndex];
            questionState = LG.QUESTIONSTATE_NOT_FINAL;
            questionIndex++;
            return currentQuestion;
        } else if (questionIndex == questionList.length - 1) {

            int size = connections.size();
            for (int i = 0; i < size; i++) {
                StudentClient con = connections.get(i);
                if (con.loginStatus()) {
                    con.setAllowedToAnswer(true);
                }
            }
            
            currentQuestion = questionList[questionIndex];
            questionState = LG.QUESTIONSTATE_FINAL_IN_ROUND;
            questionIndex++;
            return currentQuestion;
        }

        manager.setState(LG.STATE_END_OF_ROUND);
        return null;

    }

    public String createMasterQuestion() {
        return currentQuestion.makeMasterQuestion(questionState);
    }

    public String createClientQuestion() {
        return currentQuestion.makeClientQuestion(questionState);
    }

    public void initiateRound() {
        questionList = db.getQuestion(roundID);
    }

    public int getMaxScore() {
        return maxScorePerQuestion;
    }

    public String handleClientQuestionReply(int score, int roundScore, int totalPossibleScore) {

        StringBuilder s = new StringBuilder("@^RES");
        s.append(currentQuestion.getCorrectAnswer());
        s.append("|");
        s.append(score);
        s.append("|");
        s.append(totalPossibleScore);
        s.append("^@");
        return s.toString();

    }

    public int checkAns(String answer, StudentClient c) {
        String correctAns = currentQuestion.getCorrectAnswer();
        if (correctAns.equals(answer)) return 1;
        else return 0;
    }

    public int getQuestionState() {
        return questionState;
    }

    public void setQuestionState(int s) {
        questionState = s;
    }


    public String handleQuestionReply() {
        connections = new ArrayList<StudentClient>(netKeeper.getLoggedInConnections());

        StringBuilder s = new StringBuilder("@^ANS");
        int clientSize = connections.size();
        s.append(clientSize);


        for (int i = 0; i < clientSize; i++) {
            StudentClient con = connections.get(i);
            
            if (con.loginStatus()) {
                s.append("|");
                s.append(connections.get(i).getUsername());
                s.append("|");
                String answer = connections.get(i).getAnswer();
                // hvis klienten ikke har svart noe, send -1 (istedetfor "null")
                if (answer == null || answer.equals("")) answer = "-1";
                s.append(answer);

            }

        }

        s.append("^@");
        return s.toString();

    }

    public String handleEndOfRound() {
        connections = new ArrayList<StudentClient>(netKeeper.getLoggedInConnections());

        // sending  :   RND
        //              number of participants
        //              name
        //              totalScore
        //              ....
        // flgende skal skje: send runderesultater til master, ny runde m initieres. hvis ikke ny runde finnes,
        // sett state END_OF_LECTURE
        // The following shall happend: send round results to the master client 

        StringBuilder s = new StringBuilder("@^RND");
        int clientSize = connections.size();
        s.append(clientSize);


        for (int i = 0; i < clientSize; i++) {
            s.append("|");
            s.append(connections.get(i).getUsername());
            s.append("|");
            s.append(connections.get(i).getTotalScore());

        }
        s.append("^@");
        return s.toString();

    }


}
