package LectureGame.Test.network_stresstest;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 04.apr.2007
 * Time: 10:41:21
 * A playerAgent instance has the properties of a student client connected to the server, i.e. it has a username with an
 * assiociated password, it has a lecture code which it will send and it will receive and send answers to questions.
 * The agent answers the question randomly and a at a random time during or a litte after the timelimit for
 * answering has passed. This is to emulate that not all answer the question simoultanously. An improvement of this
 * class will be to have better answering heuristics such as a tendency to answer towards the end of the given
 * timelimit.
 */
public class playerAgent {
    private String name;
    private String password;
    private String lectureCode;
    private Socket sc;

    private enum WorkState {
        QUESTION_QUE_EMPTY,
        QUESTION_QUE_FULL,
    }

    private WorkState state;
    private String quedQuestion;
    private long replyTimeDelay = 0;
    private long replyTimer;

    private PrintWriter os;
    private BufferedInputStream is;
    private int protocolState;
    private int inputState;
    private int answerIndex = -1;

    /**
     * A player agent is created using by giving her a username and a password. The lecture code is hardcoded
     * @param name  username of the agent
     * @param password password
     * @see LectureGame.Test.network_stresstest.NamesPasswords
     */
    public playerAgent(String name, String password) {
        this.name = name;
        this.password = password;
        lectureCode = "IGI";
        quedQuestion = "";
        state = WorkState.QUESTION_QUE_EMPTY;

        protocolState = Constants.PROT_ENDED_STEP2;


    }

    /**
     * Connects to the server. This method must be
     * called on an agent to connect it.
     * @throws IOException
     */
    public void connect() throws IOException {
        sc = new Socket(Constants.SERVER_ADDRESS, Constants.SERVER_PORTNUMBER);
        is = new BufferedInputStream(sc.getInputStream());
        os = new PrintWriter(sc.getOutputStream());
    }

    /**
     * Ehen a client is connected, it must log in to the server using its username and password
     */
    public void login() {
        send("@^LGN" + name + "|" + password + "|" + lectureCode + "^@");
    }

    /**
     * This is the core method called from the StressTester class on the agent to make it process any availiable
     * indata from the server, or evaluate if it should answer questions
     * @throws IOException
     */
    public void doWork() throws IOException {

        switch (state) {

            case QUESTION_QUE_EMPTY:
                if (is.available() > 0) {
                    parse(receive());
                }
                break;

            case QUESTION_QUE_FULL:
                if (replyTimeDelay >= System.currentTimeMillis() - replyTimer) {
                    sendQuestion();
                }
                break;
        }
    }

    /**
     * This method is derived directly and unmodified from the receive() method in LectureGame.Core.StudentClient
     * @return the string received
     */
    private String receive() {
        String rec = "";
        int ch = 0;
        char[] tempChar = new char[1];
        while (ch != -1) {
            try {
                ch = is.read();
                tempChar[0] = (char) ch;
                String tempString = new String(tempChar);
                rec = rec + "" + new String(tempChar);
                if ((protocolState == Constants.PROT_ENDED_STEP2)) {
                    if (tempString.equals("@")) {

                        protocolState = Constants.PROT_STARTED_STEP1;
                    } else if (tempString.equals("A")) {

                        rec = "";
                        break;
                    } else {

                        rec = "";
                        break;
                    }
                } else if ((protocolState == Constants.PROT_STARTED_STEP1)) {
                    if (tempString.equals("^")) {
                        protocolState = Constants.PROT_STARTED_STEP2;
                    } else {
                        rec = "";
                        break;
                    }
                } else if (protocolState == Constants.PROT_STARTED_STEP2) {
                    if (tempString.equals("^")) {
                        protocolState = Constants.PROT_ENDED_STEP1;
                    }
                } else if ((protocolState == Constants.PROT_ENDED_STEP1)) {
                    if (tempString.equals("@")) {
                        protocolState = Constants.PROT_ENDED_STEP2;
                        break;
                    } else {
                        protocolState = Constants.PROT_STARTED_STEP2;
                    }
                }


            } catch (IOException e) {
                e.printStackTrace();
                ch = -1;

            }

        }

        if (rec.length() > 3) {
            System.err.println(rec);
            return rec.substring(2, (rec.length() - 2));

        }
        return null;
    }

    /**
     * Parses a received string. This is a modified version of the parse() method found in LectureGame.Core.StudentClient.
     * This method handles the playerAgent logic.
     * @param received
     * @throws IOException
     */
    private void parse(String received) throws IOException {

        if (received == null || received.length() == 0) return;
        int numberOfFields = 0;
        String[] stringList = new String[Constants.MAX_NUMBER_OF_FIELDS_IN_MSG];
        String code = received.substring(0, 3);
        received = received.substring(3, received.length());
        while (numberOfFields < Constants.MAX_NUMBER_OF_FIELDS_IN_MSG) {
            int msgLength = received.indexOf("|");
            if (msgLength == (-1)) {
                stringList[numberOfFields] = received.substring(0, received.length());
                break;
            } else {
                stringList[numberOfFields] = received.substring(0, Math.abs(msgLength));
                received = received.substring(msgLength + 1, received.length());
                //System.err.println(received + " " + msgLength);
                numberOfFields++;
            }
        }


        if (code.equals("CRP")) {
            send("T");

        } else if (code.equals("QST")) {
            // first field is the question (not currently used by the playerAgent)
            String question = stringList[0];
            // 2nd field is the number of alternatives
            int nbrOfAlternatives = Integer.parseInt(stringList[1]);

            // last parmeter is the timelimit for that question. 0 = default timelimit
            int seconds = Integer.parseInt(stringList[nbrOfAlternatives + 2]);
            String[] alternatives = new String[nbrOfAlternatives];
            System.arraycopy(stringList, 2, alternatives, 0, nbrOfAlternatives);

            // incredible Math.random() "AI" which makes the agent select an alternative at random
            int reply = (int) (Math.random() * nbrOfAlternatives);

            // code to handle time used to answer the question
            replyTimeDelay = Math.round(Math.random() * (seconds + 5) * 1000);

            // string to be sendt to the server
            quedQuestion = "@^ANS" + reply + "^@";
            state = WorkState.QUESTION_QUE_FULL;
            replyTimer = System.currentTimeMillis();


        } else if (code.equals("LGN")) {

            if (stringList[0].equals("OK")) {
               // possible to add handeling of a correct login.
            } else if (stringList[0].equals("UNKNOWN")) {
                send("@^REG" + name + "|" + password + "|" + lectureCode + "^@");
            } else close();

        } else if (code.equals("RES")) {

            if (answerIndex != Integer.parseInt(stringList[0])) {
                // possible to add handeling of a false answer here
            } else {
                // possible to add handeling of a correct answer here
            }

            inputState = Constants.LOGGED_IN;
        } else if (code.equals("DIE")) {
            close();
        }
    }

    /**
     * Sends a string over the socket connection
     * @param str the string to be sendt
     */
    private void send(String str) {
        os.println(str);
        os.flush();
        System.err.println("Sender " + str);
    }

    /**
     * Sends a question reply over the socket
     */
    private void sendQuestion() {
        send(quedQuestion);
        quedQuestion = "";
        state = WorkState.QUESTION_QUE_EMPTY;
    }

    /**
     * closes the socket and associated streams
     * @throws IOException
     */
    public void close() throws IOException {
        os.close();
        is.close();
        sc.close();
    }
}
