package LectureGame.gui;

import LectureGame.Core.LG;
import LectureGame.Core.NetworkManager;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 03.may.2007
 * Time: 12:12:09
 * This class creates and paints a console window, used in place of a commandline window when running the server.
 * <i>This class is currently not used by the application, and is only included for reference. For further info,
 * see the "Game Enahced Lectures" master thesis</i>
 *
 * @see LectureGame.Core.ConsoleWriter
 */
public class ConsoleScreen extends JFrame implements ClipboardOwner {

    private static SimpleDateFormat date = new SimpleDateFormat("yyyyMMdd-HHssmm");
    private static int ERROR = 5;
    private static int NORMAL = 4;
    private static int DEBUG = 3;
    private StyledDocument inBuffer;
    private NetworkManager brain = null;
    private javax.swing.JTextPane jTextPane1;
    private MutableAttributeSet currentAttrib;
    private MutableAttributeSet[] AttribSet;
    private JMenu developerMenu;
    private JMenuItem jMenuItemPing;

    /**
     * Takes in the name of the window as an argument. The name is displayed at the window title bar
     *
     * @param name window name
     */
    public ConsoleScreen(String name) {
        super(name);

        initComponents();
        inBuffer = new DefaultStyledDocument();
        jTextPane1.setStyledDocument(inBuffer);
        jTextPane1.setEditable(false);
        AttribSet = initAttributes(6);
        currentAttrib = AttribSet[1];
        setBackground(Color.white);
        SimpleAttributeSet attrs = new SimpleAttributeSet();
        StyleConstants.setAlignment(attrs, StyleConstants.ALIGN_LEFT);
        setVisible(true);
    }

    /**
     * empty method
     */
    public void handleInput() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * empty method
     */
    public void handleLogic() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * empty method
     */
    public void repaint() {
        // not implemented
    }

    /**
     * Prints error messages to the console using red, italic text.
     * @param text the error message to be printed
     */
    public void printErr(String text) {
        currentAttrib = AttribSet[ERROR];
        print(text);
    }

    /**
     * Prints normal text to the console
     * @param text the text to be printed
     */
    public void println(String text) {
        currentAttrib = AttribSet[NORMAL];
        print(text);
    }

    /**
     * Prints debug text to the console using green text
     * @param text the text to be printed
     */
    public void debug(String text) {
        currentAttrib = AttribSet[DEBUG];
        print(text);
    }

    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Sets the networkmanager for the console window
     * @deprecated
     * @param nm
     */
    public void setNetworkManager(NetworkManager nm) {
        brain = nm;
    }

    // private methods below

    /**
     * Prints text to the console. Used by the public print methods. Updates caret position
     * @param text text to be printed
     */
    private void print(String text) {

        try {
            inBuffer.insertString(inBuffer.getLength(), text + "\n", currentAttrib);
            jTextPane1.setCaretPosition(inBuffer.getLength());

        } catch (BadLocationException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Helper method which returns the SimpleAttributeSet used by the console when printing text.
     * @param length
     * @return SimpleAttributeSet[]
     */
    private SimpleAttributeSet[] initAttributes(int length) {
        //Hard-code some attributes.

        SimpleAttributeSet[] attrs = new SimpleAttributeSet[length];

        attrs[0] = new SimpleAttributeSet();
        StyleConstants.setFontFamily(attrs[0], "Monospaced");
        StyleConstants.setAlignment(attrs[0], StyleConstants.ALIGN_LEFT);
        StyleConstants.setFontSize(attrs[0], 11);


        attrs[1] = new SimpleAttributeSet(attrs[0]);
        StyleConstants.setBold(attrs[1], true);

        attrs[2] = new SimpleAttributeSet(attrs[0]);


        attrs[3] = new SimpleAttributeSet(attrs[0]);
        StyleConstants.setItalic(attrs[3], true);
        StyleConstants.setForeground(attrs[3], new Color(0x2ea82c));

        attrs[4] = new SimpleAttributeSet(attrs[0]);
        StyleConstants.setForeground(attrs[4], Color.black);

        attrs[5] = new SimpleAttributeSet(attrs[0]);
        StyleConstants.setItalic(attrs[5], true);
        StyleConstants.setForeground(attrs[5], Color.red);

        return attrs;
    }

    /**
     * Action handeling code for actions performed on jMenu1
     * @param evt
     */
    private void jMenu1ActionPerformed(java.awt.event.ActionEvent evt) {
        // TODO add your handling code here:
    }

    /**
     * Action handeling code for actions performed on MenuItemSave. Opens a save dialog for
     * saving the content of the console as a text file
     * @param evt
     */
    private void MenuItemSaveActionPerformed(java.awt.event.ActionEvent evt) {
        final JFileChooser fc = new JFileChooser();

        fc.setSelectedFile(new File(date.format(new Date()) + ".log"));

        int returnVal = fc.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            try {
                File file = fc.getSelectedFile();
                file.createNewFile();

                //lagre i filen...

                try {
                    String bufferContents = inBuffer.getText(0, inBuffer.getLength());
                    FileWriter fw = new FileWriter(file);
                    fw.write(bufferContents);
                    fw.flush();
                    fw.close();
                } catch (IOException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (BadLocationException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            } catch (IOException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
    }

    /**
     * Action handeling code for actions performed on MenuItemCopy. Copies the current content of
     * the console to the clipboard.
     * @param evt
     */
    private void MenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {

        try {
            String copy = inBuffer.getText(0, inBuffer.getLength());
            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
            StringSelection stringSelection = new StringSelection(copy);
            clipboard.setContents(stringSelection, this);

        } catch (BadLocationException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

    }

    /**
     * Action handeling code for actions performed on MenuItemPing. Sets the NetworkManager in ping mode.
     * @param evt
     * @see LectureGame.Core.NetworkManager
     */
    private void MenuItemPingActionPerformed(java.awt.event.ActionEvent evt) {
        if (brain != null) brain.setState(LG.STATE_PING);
    }



    /**
     * Helper method for initializing all GUI components. Created with netbeans and copied into this class.
     */
    private void initComponents() {
        JScrollPane jScrollPane1 = new JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        JMenuBar jMenuBar1 = new JMenuBar();
        JMenu jMenu1 = new JMenu();
        JMenuItem menuItemCopy = new JMenuItem();
        JMenuItem menuItemSave = new JMenuItem();
        JMenu jMenu2 = new JMenu();        

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("LG: Console");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        jTextPane1.setFont(new java.awt.Font("Courier New", 0, 11));
        jScrollPane1.setViewportView(jTextPane1);

        jMenuBar1.setBorderPainted(false);
        jMenu1.setMnemonic('F');
        jMenu1.setText("File");
        jMenu1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenu1ActionPerformed(evt);
            }
        });


        menuItemCopy.setText("Copy");
        menuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuItemCopyActionPerformed(evt);
            }
        });

        jMenu1.add(menuItemCopy);

        menuItemSave.setText("Save");
        menuItemSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuItemSaveActionPerformed(evt);
            }
        });

        jMenu1.add(menuItemSave);

        jMenuBar1.add(jMenu1);

        jMenu2.setMnemonic('A');
        jMenu2.setText("Help");

        jMenuBar1.add(jMenu2);

        if (LG.ENABLE_DEVELOPERMENUES) {

            developerMenu = new JMenu();
            jMenuItemPing = new JMenuItem();

            developerMenu.setMnemonic('D');
            developerMenu.setText("Developer Functions");
            jMenuItemPing.setText("Ping Clients");
            jMenuItemPing.setToolTipText("Ping connected clients. See code for ping interval settings, and detailed info");
            developerMenu.add(jMenuItemPing);

            jMenuBar1.add(developerMenu);

            jMenuItemPing.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    MenuItemPingActionPerformed(evt);
                }
            }
            );
        }  //end if developermenues


        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 765, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE)
        );
        pack();
    }
}
