package core;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;


/**
 * 
 * This class runs as a separate Thread managing network data. 
 * 
 * @author Terje
 *
 */
public class ClientNetworkThread implements Runnable {

	protected LectureGame lega;
	protected PrintWriter os;
	protected BufferedReader is;
	protected Socket socket;
	protected int protocolState;
	protected boolean stringBuffered;
	public String received;
	protected Thread networkThread;

	public ClientNetworkThread(LectureGame lega) {
		this.lega = lega;
		socket = null;
		is = null;
		os = null;
		protocolState = Constants.PROT_ENDED_STEP2;
		stringBuffered = false;
		received = null;
		networkThread = Thread.currentThread();

	}

/***
 * initiates connection to the server and constantly listens for incomming data and routs them to 
 * appropriate methods for reception and parsing.
 * 
 */
	public synchronized void run() {
		boolean soFarSoGood = true;
		try {
			socket = new Socket(Constants.SERVER_ADDRESS,Constants.SERVER_SOCKET);
			//socket.setSocketOption(SocketConnection.DELAY, 0);
			//socket.setSocketOption(SocketConnection.KEEPALIVE, 0);
		} catch (Exception e) {
			e.printStackTrace();
			soFarSoGood = false;
			lega.networkFailure();
		}


		try {

			is = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			System.err.println("inputStrem");
		} catch (Exception e) {
			e.printStackTrace();
			soFarSoGood = false;
			lega.networkFailure();
		}


		try {
			os = new PrintWriter(socket.getOutputStream(), true);
			System.err.println("OS LAGET");
		} catch (Exception e) {
			soFarSoGood = false;
			lega.networkFailure();
		}

		if (soFarSoGood)
			lega.networkStatus(true);
		else {
			lega.networkStatus(false);
			lega.networkFailure();
		}

		if(soFarSoGood){
			while (true) {
				received = receive();
				if (!(received==null)) {
					stringBuffered = true;
					try {
						lega.parse(received);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

				try {
					Thread.currentThread().sleep(Constants.SAMPLE_RATE);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			} 
		}
	}



	/**
	 * Strips off protocol specific headers from  the inputStream and returns the remaining String.
	 * Incoming data not following protocol is results in the return of the value null.
	 * 
	 * @return
	 */
	public String receive() {
		String rec = "";
		int ch = 0;
		char[] tempChar = new char[1];
		while (ch != -1) {
			try {
				ch = is.read();
				System.err.print((char)ch);
				tempChar[0] = (char) ch;
				String tempString = new String(tempChar);
				rec = rec + "" + new String(tempChar);
				if ((protocolState == Constants.PROT_ENDED_STEP2)) {
					if (tempString.equals("@")) {
						protocolState = Constants.PROT_STARTED_STEP1;
					} else {
						rec = "";
						break;
					}
				} else if ((protocolState == Constants.PROT_STARTED_STEP1)) {
					if (tempString.equals("^")) {
						protocolState = Constants.PROT_STARTED_STEP2;
					} else {
						rec = "";
						break;
					}
				} else if (protocolState == Constants.PROT_STARTED_STEP2) {
					if (tempString.equals("^")) {
						protocolState = Constants.PROT_ENDED_STEP1;
					}
				} else if ((protocolState == Constants.PROT_ENDED_STEP1)) {
					if (tempString.equals("@")) {
						protocolState = Constants.PROT_ENDED_STEP2;
						break;
					} else {
						protocolState = Constants.PROT_STARTED_STEP2;
					}
				}


			} catch (IOException e) {
				e.printStackTrace();
				ch = -1;
			}

		}
		if (rec.length() > 3) {
			return rec.substring(2, (rec.length() - 2));
		}
		return null;
	}

/***
 *  Sends a request for close message to the server which initiatez closing of the connection. 
 *
 */
	public void close(){

		try {
			this.send("@^DIE^@");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/***
	 * Sends the given String to the server.
	 * 
	 * @param str
	 */
	public void send(String str) {
		os.print(str);
		os.flush();
		System.err.println("sendt: " + str);
	}
}