package core;
import graphics.GraphicsFacade;
import graphics.EliminationMode;
import graphics.Renderer;
import graphics.Respondent;


import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;



/***
 * 
 * COntains the application entry points and features parsing and high level method calls
 * for further processing.
 * 
 * 
 * @author Terje
 *
 */
public class LectureGame {

protected boolean connectionOK;
protected ClientNetworkThread clientNetwork;
protected Thread networkThread;
protected ArrayList loggedOnUsers;
protected GraphicsFacade graphics = null;
protected int inputState;
protected JFrame loginFrame;
protected LoginPanel loginPanel;
protected int currentModeActivated = -1;
protected boolean lastQuestionInitialized = false;

	/***
	 * Initiates the program and login GUI.
	 *
	 */
public LectureGame(){
	
	if(!Constants.IS_SIMULATING){
	clientNetwork = new ClientNetworkThread(this);
    networkThread = new Thread(clientNetwork);
    networkThread.start();
	}
	inputState = Constants.STATE_LOGIN_IDLE;
	loginFrame = new JFrame("The Lecture Game");
	loginPanel = new LoginPanel(this, new GridBagLayout());
	
	loginFrame.addWindowListener( new LoginWindowAdapter() );
	loginFrame.getContentPane().add(loginPanel);
	loginFrame.setPreferredSize(new Dimension(500,400));
	loginFrame.setSize(500, 36*(loginPanel.getGridOffset()+5));
	loginFrame.setLocation(300,100);
	loginFrame.setVisible(true);
	

}


/***
 * 
 * Hides the login window, initiates the selected lecture and intitates the graphicsFacade class 
 * for future graphical processing.
 * 
 * @param isFullScreen
 * @param dimension
 * @param lectureCode
 */
public void launchGame(boolean isFullScreen, Dimension dimension, String lectureCode){
	if(!(clientNetwork==null))
	clientNetwork.send("@^LTR" + lectureCode + "^@");
	loginFrame.setVisible(false);
	loginPanel.setVisible(false);
	loginFrame.dispose();
	graphics = new GraphicsFacade(isFullScreen,dimension,this);
	graphics.setLectureCode("M");
	if(Constants.IS_SIMULATING){
		InputSimulatorThread sim = new InputSimulatorThread(this);
		Thread simThread = new Thread(sim);
		simThread.start();
	}
		
}

/***
 * 
 * Called before posing the last question in order for the logic of the program to know when to display
 * the high score list. 
 * 
 * @param isInitialized
 */
public void setLastQuestionInitialized(boolean isInitialized){
	lastQuestionInitialized = isInitialized;
}

/***
 * 
 * Sets the current gameMode. Graphics are rendered and result messages are interpreted accoring to this parameter.
 * 
 * 
 * @param mode
 */
public void setGameMode(int mode){
	while(true)
		if(graphics!=null){
			graphics.getRenderer().setMode(mode);
			break;
		}
		
}

/***
 * 
 * Returns the integer indicatring the current game mode.
 * 
 * @return
 */
public int getGameMode(){
	return currentModeActivated;
}

/**
 * 
 * This method is called if the network connection is not properly initiated. 
 * For debugging purposes.
 *
 */
public void networkFailure(){
	System.err.println("NETVERKSOPPKOBLING FEILET");
	
}

/***
 * Parses the input strings nad calls the appropriate method with appopriate inout.
 * 
 * 
 * @param received
 */
public void parse(String received) {
    int numberOfFields = 0;
    String[] stringList = new String[Constants.MAX_NUMBER_OF_FIELDS_IN_MSG];
    String code = received.substring(0, 3);
    received = received.substring(3, received.length());
    while (numberOfFields < Constants.MAX_NUMBER_OF_FIELDS_IN_MSG) {
        int msgLength = received.indexOf("|");
        if (msgLength == (-1)) {
            stringList[numberOfFields] = received.substring(0, received.length());
            numberOfFields++;
            break;
        } else {
            stringList[numberOfFields] = received.substring(0, Math.abs(msgLength));
            received = received.substring(msgLength + 1, received.length());
            //System.err.println(received + " " + msgLength);
            numberOfFields++;
        }
    }
    //for (int k = 0; k < numberOfFields; k++) {
    //    System.err.println(stringList[k]);
    //}
    if (code.equals("QST")) {
        String question = stringList[0];
        int nbrOfAlternatives = Integer.parseInt(stringList[1]);
        ArrayList<String> alternatives = new ArrayList<String>();
        for (int i = 0; i < nbrOfAlternatives; i++) {
            alternatives.add(stringList[i + 2]);
        }
        int seconds = Integer.parseInt(stringList[nbrOfAlternatives + 2]);
        int correctAlternativeIndex = Integer.parseInt(stringList[nbrOfAlternatives + 3]);
        int controlCommand = (Integer.parseInt(stringList[nbrOfAlternatives + 4]));

        graphics.newQuestion(question, alternatives, seconds,correctAlternativeIndex, controlCommand);
    } else if (code.equals("LGN")) {
    	System.err.println("LOGIN OK");
    	ArrayList<String> lectureList = new ArrayList<String>();
    	ArrayList<String> codeList = new ArrayList<String>();
    	int nbrOfLectures = Integer.parseInt(stringList[0]);
    	for(int i=0;i<nbrOfLectures;i++){
    		lectureList.add(stringList[(i*3)+1] + " : " + stringList[(i*3)+2]);
    		codeList.add(stringList[(i*3)+3]);
    		
    	}
    	loginPanel.setComboModel(new LectureListModel(lectureList, codeList));
        loginPanel.loginOK(lectureList);
    } else if (code.equals("ANS")) {
    	int nbrOfRespondents = Integer.parseInt(stringList[0]);
         Respondent[] respondents = new Respondent[nbrOfRespondents];
         for (int i = 0; i < nbrOfRespondents; i++) {
        	 try{
             respondents[i] = new Respondent(stringList[(i*2)+1],Integer.parseInt(stringList[(i*2)+2]));
        	 }catch(Exception efd){
            	 
             }
         }
         graphics.returnResults(respondents);
    }
    else if (code.equals("RND")) {
    	int nbrOfRespondents = Integer.parseInt(stringList[0]);
    	Respondent[] respondents = new Respondent[nbrOfRespondents];
    	for (int i = 0; i < nbrOfRespondents; i++) {
    		respondents[i] = new Respondent(stringList[(i*2)+1],Integer.parseInt(stringList[(i*2)+2]));
    	}
    	graphics.returnRoundResults(respondents);
    }
    else if (code.equals("LCT")) {
    	int nbrOfRespondents = Integer.parseInt(stringList[0]);
    	Respondent[] respondents = new Respondent[nbrOfRespondents];
    	for (int i = 0; i < nbrOfRespondents; i++) {
    		respondents[i] = new Respondent(stringList[(i*2)+1],Integer.parseInt(stringList[(i*2)+2]));
    	}
    	graphics.returnGameResults(respondents);
    }  else if (code.equals("MOD")) {
    	this.setGameMode(Integer.parseInt(stringList[0]));
    } else if (code.equals("USR")){
    	if(!(graphics==null))
    	graphics.userLoggedIn(stringList[0]);
    }
    clientNetwork.received = null;
}

public void login(String username, String password){
	if(!(clientNetwork==null))
	clientNetwork.send("@^LGN" + username + "|" + password + "^@");
}

public void notifyReadyToReceive(){
	//clientNetwork.wakeMe();
}

public void networkStatus(boolean status){
	
}

	public static void main(String[] args) {
		 	LectureGame LG = new LectureGame();

	}

    public ClientNetworkThread getNetworkConnection(){
    	return clientNetwork;
    }

	public void netSend(String string) {
		if(!(clientNetwork==null)){
		clientNetwork.send(string);
		} else {
			System.err.println("FORSKT SENDT:" +string);
		}
		
	}
	
	private class LoginWindowAdapter extends WindowAdapter {
	    public void windowClosing( WindowEvent e ) {
	    	try{
	    		clientNetwork.send("@^DIE^@");
	    	} catch(Exception e1){
	    		
	    	}
	    }
	}
	
	
}
