package core;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.event.ListDataListener;



/**
 * 
 * The Login panel for Teacher login.
 * 
 * @author Terje
 *
 */
public class LoginPanel extends JPanel implements ActionListener {

	JLabel usernameLabel;
	JTextField textField;
	JLabel passLabel;
	JPasswordField passField;
	JButton button;
	LectureGame lega;
	GridBagConstraints c;
	JComboBox combo;
	JButton launchFButton; 
	JButton launchWButton; 
	ButtonGroup bg;
	
	ArrayList<String> lectures;
	int gridOffset = 0;
	
	public LoginPanel(LectureGame lega, LayoutManager layout){
		super(layout);
		this.lega = lega;
		combo = new JComboBox();
		
		c = new GridBagConstraints();
		usernameLabel = new JLabel("Username: ");
		textField = new JTextField(20);
		passLabel = new JLabel("Password: ");
		passField = new JPasswordField(20);
		passField.addActionListener(this);

		button = new JButton("OK");
		button.addActionListener(this);

		c.gridx = 0; c.gridy = 0;
		this.add(usernameLabel,c);
		c.gridx = 1;
		this.add(textField,c);
		c.gridx=0; c.gridy = 1;
		this.add(passLabel,c);
		c.gridx = 1;
		this.add(passField,c);
		
		c.insets = new Insets(10,0,0,0); 
		GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		DisplayMode[] modes = device.getDisplayModes();
		bg = new ButtonGroup();
		ArrayList<Dimension> uniqueModes = new ArrayList<Dimension>();
		for(int i=0;i<modes.length;i++){
			Dimension temp = new Dimension(modes[i].getWidth(),modes[i].getHeight());
			if((!uniqueModes.contains(temp))&&(temp.width>=800)){
				uniqueModes.add(temp);
			}
		}
		
		c.gridx = 0; c.gridy = 2;
		this.add(new JLabel("Select resolution:"),c);
		for(int i=0;i<uniqueModes.size();i++){
			c.gridx = 1; c.gridy = i+2;
			JRadioButton tempRadio = new JRadioButton(""+ uniqueModes.get(i).width + "x" + uniqueModes.get(i).height);
			bg.add(tempRadio);
			this.add(tempRadio,c);
		}

		c.gridx = 1; c.gridy = uniqueModes.size()+3;
		
		c.insets = new Insets(10,0,0,0); 
		this.add(button,c);
		gridOffset = uniqueModes.size() + 3;
		
		
		
	}
	
	public void loginOK(ArrayList<String> lectures){
		this.lectures = lectures;
		
		c.gridx = 1; c.gridy = gridOffset+1;
		c.insets = new Insets(10,0,0,0); 
		c.gridx = 0; 
		this.add(new JLabel("Select Lecture :"),c);

		
		c.gridx = 1;
		this.add(combo,c);

		launchFButton = new JButton("Launch Fullscreen");
		launchWButton = new JButton("Launch Windowed");
		launchFButton.addActionListener(this);
		launchWButton.addActionListener(this);
		c.insets = new Insets(30,0,0,0); 
		c.gridx=0; c.gridy=gridOffset+2;
		this.add(launchFButton,c);
		c.gridx=1; c.gridy=gridOffset+2;
		this.add(launchWButton,c);
		this.paintAll(this.getGraphics());
		this.setVisible(true);
		
		
		
	}
	
	public void loginNotOK(){
		c.gridx = 1; c.gridy = gridOffset+1;
		c.insets = new Insets(10,0,0,0); 
		c.gridx = 0; 
		
		this.add(new JLabel("Login failed!"),c);
		this.paintAll(this.getGraphics());
		this.setVisible(true);
	}
	
	
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("OK")){
			//send login sjekk om den gr gjennom
			ArrayList<String> temp = new ArrayList<String>();
			temp.add("TDT4142, Patterns");
			setComboModel(new LectureListModel(temp, temp));
			//temp.add("TDT4142, Semaphors and Metaphors");
			if(!Constants.IS_SIMULATING){
				lega.login(textField.getText(),passField.getText());
			}
			loginOK(temp);
				
		} else {
			Dimension tempDim = null;
			JRadioButton tempButt;
			Enumeration<AbstractButton> enume = bg.getElements();
			for(int i=0;i<bg.getButtonCount();i++){
				tempButt = ( (JRadioButton) enume.nextElement());
				if(tempButt.isSelected()) {
					tempDim = new Dimension(Integer.parseInt(tempButt.getText().substring(0,tempButt.getText().indexOf("x"))),Integer.parseInt(tempButt.getText().substring(tempButt.getText().indexOf("x")+1, tempButt.getText().length())));
					break;
				}
			}
		if(e.getActionCommand().equals("Launch Fullscreen")){
			
			lega.launchGame(true,tempDim, ((LectureListModel)combo.getModel()).getLectureCode(combo.getSelectedIndex()));
			lega.notifyReadyToReceive();
		} else if(e.getActionCommand().equals("Launch Windowed")){
			lega.launchGame(false,tempDim, ((LectureListModel)combo.getModel()).getLectureCode(combo.getSelectedIndex()));
			lega.notifyReadyToReceive();
		}
		}
	}

	public void setComboModel(LectureListModel listModel){
		combo.setModel(listModel);		
	}

	public int getGridOffset(){
		return gridOffset;
	}
	
	
}

