package graphics;

import java.io.IOException;
import java.util.TimerTask;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;

/**
 * Represent a graphical bar used in the Measure Up Mode. After setting the start drawing point in the openGL
 * context and instantiating a bar. The bar can be drawn.
 * 
 * @author Terje
 *
 */
public class BarGraphics {

	private float maxHeight = 5.0f;
	private float currentHeigth = 0;
	private float width = 1.0f;
	private float stopHeight = 0.0f;
	private boolean barStopped = false;
	private float flyHeigth = 0.0f;
	private String feedback = "";
	private float feedBackAlpha = 1.0f;
	private float r;
	private float g;
	private float b;
	private boolean revealAnswer = false;
	private boolean blinkOn = false;
	private AudioSample powerupSound;
	private AudioSample powerDownSound;
	private int pourcentageScore = 0;

	
	/**
	 * 
	 * Initiates a new bar.
	 * 
	 * @param r Amount of red.
	 * @param g Amount of g.
	 * @param b AMount of blue
	 * @param stopHeight heigth of the bar when it is to stop growing.
	 */
	public BarGraphics(float r, float g, float b, double stopHeight){
		this.r = r;
		this.g = g;
		this.b = b;
		this.setStopHeight((int)stopHeight);
		try {
			powerupSound = new  AudioSample(ResourceRetriever.getResourceAsStream("Powerup.wav"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			powerDownSound = new  AudioSample(ResourceRetriever.getResourceAsStream("Sonicdie.wav"));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @return wether or not the correct answer is ready to be revealed.
	 */
	 public boolean isRevealAnswer() {
		return revealAnswer;
	}

	 /**
	  * Performs the acutal drawing of the bar on the OpenGL canvas.
	  * 
	  * @param glDrawable
	  */
	public void renderBar(GLAutoDrawable glDrawable){
	   		if(!blinkOn){
		 	final GL gl = glDrawable.getGL();
	        gl.glBegin(GL.GL_QUADS);           	// Draw A Quad
	        gl.glColor3f(r*0.7f, g*0.7f, b*0.7f);
	        gl.glVertex3f(1.0f, getCurrentHeigth(), -1.0f);			// Top Right Of The Quad (Top)
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), -1.0f);			// Top Left Of The Quad (Top)
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), 1.0f);			// Bottom Left Of The Quad (Top)
	        gl.glVertex3f(1.0f, getCurrentHeigth(), 1.0f);			// Bottom Right Of The Quad (Top)

	        gl.glColor3f(r, g, b);
	        gl.glVertex3f(1.0f, -0.2f, 1.0f);			// Top Right Of The Quad (Bottom)
	        gl.glVertex3f(-1.0f, -0.2f, 1.0f);			// Top Left Of The Quad (Bottom)
	        gl.glVertex3f(-1.0f, -0.2f, -1.0f);			// Bottom Left Of The Quad (Bottom)
	        gl.glVertex3f(1.0f, -0.2f, -1.0f);			// Bottom Right Of The Quad (Bottom)

	        gl.glColor3f((r*0.7f)+ 0.3f,(g*0.7f)+ 0.3f, (b*0.7f)+ 0.3f);			// Set The Color To Red
	        gl.glVertex3f(1.0f, getCurrentHeigth(), 1.0f);			// Top Right Of The Quad (Front)
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), 1.0f);			// Top Left Of The Quad (Front)
	        gl.glColor3f(r, g, b);
	        gl.glVertex3f(-1.0f, -0.2f, 1.0f);			// Bottom Left Of The Quad (Front)
	        gl.glVertex3f(1.0f, -0.2f, 1.0f);			// Bottom Right Of The Quad (Front)

	        gl.glColor3f(r, g, b);
	        gl.glVertex3f(1.0f, -0.2f, -1.0f);			// Bottom Left Of The Quad (Back)
	        gl.glVertex3f(-1.0f, -0.2f, -1.0f);			// Bottom Right Of The Quad (Back)
	        gl.glColor3f((r*0.7f)+ 0.3f, (g*0.7f)+ 0.3f, (b*0.7f)+ 0.3f);	
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), -1.0f);			// Top Right Of The Quad (Back)
	        gl.glVertex3f(1.0f, getCurrentHeigth(), -1.0f);			// Top Left Of The Quad (Back)

	        gl.glColor3f((r*0.7f)+0.3f, (g*0.7f)+0.3f, (b*0.7f)+0.3f);			// Set The Color To Blue
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), 1.0f);			// Top Right Of The Quad (Left)
	        gl.glVertex3f(-1.0f, getCurrentHeigth(), -1.0f);			// Top Left Of The Quad (Left)
	        gl.glColor3f(r, g, b);
	        gl.glVertex3f(-1.0f, -0.2f, -1.0f);			// Bottom Left Of The Quad (Left)
	        gl.glVertex3f(-1.0f, -0.2f, 1.0f);			// Bottom Right Of The Quad (Left)

	        gl.glColor3f(r*0.3f, g*0.3f, b*0.3f);			// Set The Color To Violet
	        gl.glVertex3f(1.0f, getCurrentHeigth(), -1.0f);			// Top Right Of The Quad (Right)
	        gl.glVertex3f(1.0f, getCurrentHeigth(), 1.0f);			// Top Left Of The Quad (Right)
	        gl.glColor3f(r*0.7f, g*0.7f, b*0.7f);			// Set The Color To Violet
	        gl.glVertex3f(1.0f, -0.2f, 1.0f);			// Bottom Left Of The Quad (Right)
	        gl.glVertex3f(1.0f, -0.2f, -1.0f);			// Bottom Right Of The Quad (Right)
	        gl.glEnd();				// Done Drawing The Quad

	   		}
	        if(getCurrentHeigth()<getStopHeight()){

	        } else {
	        	setBarStopped(true);
	        }
	    	updateState();
	    }
	
	/**
	 * Called to for each frame to perform the actual grwoing of the bar as well as the blinking.
	 *
	 */
	public void updateState(){
		if(!barStopped)
			currentHeigth+=0.02f;
		else if(revealAnswer ){
			flyHeigth+= 0.02f;
			feedBackAlpha -= 0.004f;
			if(System.currentTimeMillis()%600>300)
				blinkOn = true;
			else
				blinkOn = false;
		}
	}

	/**
	 * resets the local counters of the bar.
	 *
	 */
	public void reset(){

		currentHeigth = 0.0f;
		flyHeigth = 0.0f;
		feedBackAlpha = 1.0f;
		barStopped = false;
	}
	
	/**
	 * 
	 * @return the current height of the drawn bar.
	 */
	public float getCurrentHeigth() {
		return currentHeigth;
	}

	/**
	 * Used to increase the hight of the bar in real time as the bar grows.
	 * 
	 * @param currentHeigth the wanted heigth of the bar. 
	 */
	public void setCurrentHeigth(float currentHeigth) {
		this.currentHeigth = currentHeigth;
	}

	public float getMaxHeigth() {
		return maxHeight;
	}

	public void setMaxHeigth(float maxHeight) {
		this.maxHeight = maxHeight;
	}

	public float getWidth() {
		return width;
	}

	public String getFeedback() {
		return feedback;
	}

	public void setFeedback(String feedback) {
		this.feedback = feedback;
	}

	public void setWidth(float width) {
		this.width = width;
	}

	public float getStopHeight() {
		return stopHeight;
	}

	public void setStopHeight(int pourcentageScore) {
		this.stopHeight = (((float)pourcentageScore)/100)*maxHeight;
		this.flyHeigth = this.stopHeight;
		this.pourcentageScore = pourcentageScore;
		if(pourcentageScore>50)
			feedback = "Good job!";
		else if(pourcentageScore<27){
			feedback = "Awful!";
		} else {
			feedback = "Mediocre!";
		}
	}
	
	public int getDrawnScore(){
		return (int)((currentHeigth/maxHeight)*100);
	}
  
	public boolean isBarStopped() {
		return barStopped;
	}

	public void setBarStopped(boolean barStopped) {
		this.barStopped = barStopped;
	}

	public float getFlyHeigth() {
		return flyHeigth;
	}

	public float getFeedBackAlpha() {
		return feedBackAlpha;
	}

	public void setFeedBackAlpha(float feedBackAlpha) {
		this.feedBackAlpha = feedBackAlpha;
	}

	public void setRevealAnswer(boolean revealAnswer) {
		this.revealAnswer = revealAnswer;
	}

	/**
	 * Play an appropriate sound relating to the score.
	 *
	 */
	public void playSound() {
		if(pourcentageScore >=50)
			powerupSound.play();
		else
			powerDownSound.play();
		
	}
	
}
