package graphics;

import graphics.TextureReader.Texture;
import java.util.Arrays;
import java.awt.Font;
import java.io.IOException;
import java.util.ArrayList;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.glu.GLU;

import com.sun.opengl.util.j2d.TextRenderer;

/**
 * Draws the feedback graphics when Elimination mode is enabled.
 * 
 * @author Terje
 *
 */
public class EliminationMode implements GameMode {

	private GLU glu = new GLU();
	ArrayList<Buddy> buddyList = new ArrayList<Buddy>();
	Buddy[][] buddyMatrix;
	int size = 0;
	float buddyGap = 3.0f;
	float startPoint = 0;
	private boolean done = false;
	private AudioSample shot;
	private long timerStart = 0;
	private TextRenderer textRenderer;
	private TextRenderer loResTextRenderer;
	private TextRenderer hiresTextRenderer;
	private int textureholder;
	TextureReader.Texture buddyTexture = null;
	private ArrayList<String> rightBuddies = new ArrayList<String>();
	private ArrayList<String> wrongBuddies = new ArrayList<String>();
	private boolean onlyWinnersLeft = false;
	private long roundInfoStartTime = 0;
	
	private float rotation = 0.0f;
	private boolean startKilling = false;
	private int killcounter = 0;
	TextureReader.Texture buddyBlinkTexture;
	TextureReader.Texture buddyKilledTexture;
	TextureReader.Texture winnerTexture;
	private String roundFeedback = "";
	private Respondent[] roundRespondents;
	private boolean continueRequested = false;
	private boolean rotateback = false;	
	
	/**
	 * Elimination mode initialization
	 *
	 */
	public EliminationMode(){
	    	try {
				shot = new  AudioSample(ResourceRetriever.getResourceAsStream("shotgun.wav"));
			} catch (IOException e) {
				e.printStackTrace();
			}
			hiresTextRenderer = new TextRenderer(new Font("Trebuchet MS", Font.BOLD, 90));
			textRenderer = new TextRenderer(new Font("Trebuchet MS", Font.PLAIN, 32));
			loResTextRenderer = new TextRenderer(new Font("Trebuchet MS", Font.PLAIN, 16));
	}
	
	/**
	 * 
	 * kills the avatar with the provided username
	 * 
	 * @param userName
	 */
	private void kill(String userName){
		timerStart = System.currentTimeMillis();
		for(int i=0;i<buddyList.size();i++){
			if(buddyList.get(i).getName().equals(userName)){
				buddyList.get(i).kill();
				buddyList.remove(i);
				shot.play();
				break;
			}
		}
	}
	
	/**
	 * Returns true if there is nothing more to draw.
	 * 
	 * @return
	 */
	public boolean isDone(){
		return done;
	}
	
	public void update(){
		
	}

/**
 * implemented from GameMode
 */
	public boolean displayFeedback(GLAutoDrawable glDrawable, boolean isLastQuestion) {
		GL gl = glDrawable.getGL();
		gl.glLoadIdentity();
		if(onlyWinnersLeft){
			if(System.currentTimeMillis()%1500>250){
				hiresTextRenderer.begin3DRendering();
				hiresTextRenderer.draw3D(roundFeedback, -3.0f, -4.0f, -10.5f, 0.006f);
				hiresTextRenderer.end3DRendering();
			}
		}

		if(buddyList.size()<3){
		gl.glTranslatef(0.0f, -1.0f, (float) -((Math.sqrt((double)wrongBuddies.size()+rightBuddies.size()))*5)-5);
		} else {
		gl.glTranslatef(0.0f, 0.0f, ((float) -((Math.sqrt((double)buddyList.size()))*6)-5));
		}
		gl.glRotatef(rotation, 1.0f, 1.0f, 0.0f);
		//gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
		makeRGBTexture(gl, glu, buddyTexture, GL.GL_TEXTURE_2D, false);

		
		if (timerStart!=0) {
			if((!startKilling)&&((System.currentTimeMillis()-timerStart)>3000)&&(!onlyWinnersLeft))
				startKilling = true;
			else if((killcounter<wrongBuddies.size())&&((System.currentTimeMillis()-timerStart)>500)&&(startKilling)) {
				kill(wrongBuddies.get(killcounter));
				killcounter++;
			}
			else if(((System.currentTimeMillis()-timerStart)>30000)||(continueRequested)){
				continueRequested = false;
				return false;
			}
			else if(!onlyWinnersLeft){

				//if((isLastQuestion&&(killcounter==wrongBuddies.size()))||((wrongBuddies.size()+rightBuddies.size())<2)||(rightBuddies.size()<2)){
				if(killcounter==wrongBuddies.size()){
					if((rightBuddies.size()<2)||(isLastQuestion)){
						onlyWinnersLeft = true;
						timerStart += 2000;
						//spill av en fet vinnerlyd
						if((rightBuddies.size())==1)
							roundFeedback = "LAST MAN STANDING!";
						else if(rightBuddies.size()>1)
							roundFeedback = "PRAISE THE ROUND WINNERS!";
						else if(rightBuddies.size()==0)
							roundFeedback = "SUCKERS! You're all dead!";
						}

				}
			}

		} else {
			timerStart = System.currentTimeMillis();
		}

		gl.glTranslatef(-startPoint, startPoint, 0.0f);
		boolean resetTex = false;
		int y = 0;
		while(y<size){
			for(int x=0;x<size;x++){
				if(!(buddyMatrix[x][y]==null)){
					if(buddyMatrix[x][y].isKilled()){
						gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
						makeRGBTexture(gl, glu, buddyKilledTexture, GL.GL_TEXTURE_2D, false);
						resetTex = true;
					} else if(onlyWinnersLeft){
						gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
						makeRGBTexture(gl, glu, winnerTexture, GL.GL_TEXTURE_2D, false);
					} else if(buddyMatrix[x][y].checkBlink()){
						gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
						makeRGBTexture(gl, glu, buddyBlinkTexture, GL.GL_TEXTURE_2D, false);
						resetTex = true;
					}
					buddyMatrix[x][y].display(gl, loResTextRenderer);
					gl.glTranslatef(buddyGap, 0.0f, 0.0f);
					if(resetTex){
						gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
						makeRGBTexture(gl, glu, buddyTexture, GL.GL_TEXTURE_2D, false);
						resetTex = false;
					}
				}
			}
			gl.glTranslatef(-(buddyGap*size), -buddyGap, 0.0f);
			y++;
		}
		if(rotation<-20f){
			rotateback = false;
		}
		if(rotation<37){
			if(rotateback){
				rotation -= 0.12f;
			} else {
				
			}
			rotation+= 0.06f;
		} else {
			rotateback  = true;
			rotation -= 0.07f;
			
		}


		return true;
	}

/**
 * Implemented from GameMode
 */
	public boolean displayRoundIntro(GLAutoDrawable glDrawable) {
		if((System.currentTimeMillis()-roundInfoStartTime)<12000){
		hiresTextRenderer.begin3DRendering();
		hiresTextRenderer.draw3D("Eliminator Round", -8.0f, 6.0f, -18.0f, 0.01f);
		hiresTextRenderer.end3DRendering();
		
		
		String desc = "The rules are simple: A single wrong answer and you're out. Every round you survive will pay off in points. If you succeed you'll be the last man standing! Heads up! ";
		textRenderer.begin3DRendering();
		
		int offset = 1;
		while(desc.length()>60){
			 int cutIndex = desc.substring(0,60).lastIndexOf(" ");
			textRenderer.draw3D(desc.substring(0,cutIndex), -8.0f, 5.0f- (1.1f*offset), -18.0f, 0.02f);
			 desc = desc.substring(cutIndex+1,desc.length());
			 offset++;
		}
		textRenderer.draw3D(desc, -8.0f, 5.0f- (1.1f*offset), -18.0f, 0.02f);
		textRenderer.end3DRendering();

		return true;
		} else return false;
	}


	public void initFeedback(QuestionGraphics question, Respondent[] buddies, GLAutoDrawable glDrawable) {
		buddyMatrix = null;
		size = 0;
		buddyGap = 3.0f;
		startPoint = 0;
		done = false;
		timerStart = 0;
		buddyList = new ArrayList<Buddy>();
		rightBuddies = new ArrayList<String>();
		wrongBuddies = new ArrayList<String>();
		timerStart = 0;
		rotation = 0.0f;
		startKilling = false;
		killcounter = 0;
		onlyWinnersLeft = false;
		
		
		
		for(int i=0;i<buddies.length;i++){
			if(buddies[i]==null){
				System.out.println("XXXX breaker lkke" + i);
				break;
			}
			System.out.print(i + ",");
			buddyList.add(new Buddy(buddies[i].getName()));
			if(buddies[i].getPoints()==0)
				wrongBuddies.add(buddies[i].getName());
			else if(buddies[i].getPoints()==1)
				rightBuddies.add(buddies[i].getName());
		}
		
		GL gl = glDrawable.getGL();

		try {
			buddyTexture = TextureReader.readTexture("buddyTex.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		
		try {
			winnerTexture = TextureReader.readTexture("buddyTexWinner.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		
		try {
			buddyBlinkTexture = TextureReader.readTexture("buddyTexblink.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		
		try {
			buddyKilledTexture = TextureReader.readTexture("buddyTexKilled.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		
		gl.glBindTexture(GL.GL_TEXTURE_2D, textureholder);
		makeRGBTexture(gl, glu, buddyTexture, GL.GL_TEXTURE_2D, false);

		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
		
		//sortere eller scramble rekkeflgen.
		
		
		
		double sqrt = (Math.sqrt((double)buddyList.size()));
		
		if(sqrt-(((int)sqrt))==0){
			size = (int) sqrt;
		} else {
			size =(int)(sqrt+1);
		}
		buddyMatrix= new Buddy[size][size];
		int i=0;
		for(int y=0;y<size;y++){
			for(int x=0;x<size;x++){
				if(i<buddyList.size()){
				buddyMatrix[x][y] = buddyList.get(i);
				buddyMatrix[x][y].setGridXY(x,y);
				}
				i++;
			}
		}
		startPoint = buddyGap*(size/2);
		
	}


	public void initMode(TextRenderer textRenderer) {
		//this.textRenderer = textRenderer;
		TextureReader.Texture texture = null;
		
	}

/**
 * Implemented from GameMode
 */
	public void initRoundIntro() {
		roundInfoStartTime = System.currentTimeMillis();
		
	}

	/**
	 * Implemented from GameMode
	 */
	public void resetFeedBack(GLAutoDrawable glDrawable) {
		wrongBuddies = null;
		rightBuddies = null;
		
	}
	
	/**
	 * 
	 * Prepares a texture for drawing.
	 * 
	 * @param gl
	 * @param glu
	 * @param img
	 * @param target
	 * @param mipmapped
	 */
	private void makeRGBTexture(GL gl, GLU glu, TextureReader.Texture img, int target, boolean mipmapped) {
		if (mipmapped) {

			glu.gluBuild2DMipmaps(target, GL.GL_RGB, img.getWidth(), img.getHeight(), GL.GL_RGB, GL.GL_UNSIGNED_BYTE, img.getPixels());
		} else {
			gl.glTexImage2D(target, 0, GL.GL_RGB, img.getWidth(), img.getHeight(), 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE, img.getPixels());
		}
	}

	/**
	 * Implemented from GameMode. EMpty method as theres is no use tp update the respondents.
	 */
	public void setRoundRespondents(Respondent[] respondents) {
		
	}

	/**
	 * 
	 * Implemented from GameMode
	 * 
	 */
	public void nextStep() {
		continueRequested  = true;
		
	}
}
