package graphics;

import javax.media.opengl.GLAutoDrawable;

import com.sun.opengl.util.j2d.TextRenderer;
/**
 * This interface defines what methods need to be implemented to incorporate new game modes into 
 * the game 
 * @author Terje
 *
 */
public interface GameMode {

	/**
	 * Calld before starting to display a new mode. Can be used to load textures, initialize 
	 * counters and reset parameters.
	 * 
	 * @param textRenderer a TextRenderer object can be optionally provided for use in later methods.
	 */
	public void initMode(TextRenderer textRenderer);
	
	/**
	 * Method is called prior to every round. Optional implementation of texture loading, counter reset
	 * or variable implementation can be added in this method.
	 *
	 */
	public void initRoundIntro();
	
	/**
	 * 
	 * Displays the actual intro to the following round. The method is continuously
	 *  called per frame until it returns false. The developper may freely use the previously initialized 
	 *  textRenderer or draw as he wishes on the glDrawable object. 
	 * 
	 * 
	 * @param glDrawable
	 * @return
	 */
	public boolean displayRoundIntro(GLAutoDrawable glDrawable);
	
	/**
	 * Method is called prior to every time feedback is shown. Optional implementation of texture loading, counter reset
	 * or variable implementation can be added in this method.
	 *
	 */
	public void initFeedback(QuestionGraphics question, Respondent[] respondents,GLAutoDrawable glDrawable);
	
	/**
	 *  Displays the actual feedback graphics. The method is continuously
	 *  called per frame until it returns false. The developper may freely use the previously initialized 
	 *  textRenderer or draw as he wishes on the glDrawable object. 
	 *  
	 * @param glDrawable
	 * @param isLastQuestion boolean telling wether the recently posed question is the last in a round.
	 * @return
	 */
	public boolean displayFeedback(GLAutoDrawable glDrawable, boolean isLastQuestion);
	
	/**
	 * Called once after displayFeedback returns. The method is intended for clenup and resetting of variables after 
	 * the feedback graphics is displayed.  
	 * 
	 * @param glDrawable
	 */
	public void resetFeedBack(GLAutoDrawable glDrawable);
	
	/**
	 * 
	 * This method is called every time a feedbakc from the server is received. Thei mplementation of this 
	 * method is meant to update the internal model of the respondents and their game mode specific response number.
	 * 
	 * 
	 * @param respondents Array of Respondent object containing their game mode specific integer e.g. last answer.
	 */
	public void setRoundRespondents(Respondent[] respondents);

	/**
	 * 
	 * This method is called when the user wishes to conticue to the next step. Normally this should result in the dispalyFeedback method to return. 
	 *
	 */
	public void nextStep();
	
}
