package graphics;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Arrays;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;

import com.sun.opengl.util.j2d.TextRenderer;

public class MeasureUpMode implements GameMode{
	private float rtri = 0.0f;
	private float correctZ = 0.0f;
	private TextRenderer textRenderer;
	private boolean done = false;
	private String correctAnswer = "";
	private ArrayList<BarGraphics> barList;
	private ArrayList<Color> colorList;
	private QuestionGraphics question = null;
	private long roundInfoStartTime = 0;
	private TextRenderer hiresTextRenderer;
	private Respondent[] roundRespondents = null;
	private boolean allBarsStoppedAssumer = false;
	private boolean soundPlayed = false;
	private boolean continueRequested = false;
	
	public MeasureUpMode(){
		hiresTextRenderer = new TextRenderer(new Font("Trebuchet MS", Font.BOLD, 90));
		textRenderer = new TextRenderer(new Font("Trebuchet MS", Font.PLAIN, 32));
	}

	public void initMode(TextRenderer textRenderer){
		barList = new ArrayList<BarGraphics>();
		colorList = new ArrayList<Color>();
		colorList.add(new Color(1.0f, 0.0f, 0.0f));
		colorList.add(new Color(0.0f, 1.0f, 0.0f));
		colorList.add(new Color(0.0f, 0.0f, 1.0f));
		colorList.add(new Color(1.0f, 1.0f, 0.0f));
		colorList.add(new Color(1.0f, 0.0f, 1.0f));
		colorList.add(new Color(0.0f, 1.0f, 1.0f));
	}

	public void initRoundIntro() {		
		roundInfoStartTime = System.currentTimeMillis();
	}
	
	public boolean displayRoundIntro(GLAutoDrawable glDrawable) {
		if((System.currentTimeMillis()-roundInfoStartTime )<12000){
			hiresTextRenderer.begin3DRendering();
			hiresTextRenderer.draw3D("Round: Measure up", -8.0f, 6.0f, -18.0f, 0.01f);
			hiresTextRenderer.end3DRendering();
			
			
			String desc = "It's every man for himself, now. Compete against your classmates by answering before the time is up. Slow and fast answers are equally rewarded. Here we go!! ";
			textRenderer.begin3DRendering();
			
			int offset = 1;
			while(desc.length()>60){
				 int cutIndex = desc.substring(0,60).lastIndexOf(" ");
				textRenderer.draw3D(desc.substring(0,cutIndex), -8.0f, 5.0f- (1.1f*offset), -18.0f, 0.02f);
				 desc = desc.substring(cutIndex+1,desc.length());
				 offset++;
			}
			textRenderer.draw3D(desc, -8.0f, 5.0f- (1.1f*offset), -18.0f, 0.02f);
			textRenderer.end3DRendering();
			return true;
			} else return false;
	}


	public void initFeedback(QuestionGraphics question,Respondent[] respondents,GLAutoDrawable glDrawable) {
		this.question = question;
		soundPlayed = false;
		double[] results = new double[6];
		System.out.println("initFeedback respondents: " +respondents.length );

		for(int i=0;i<respondents.length;i++){
			if(!(respondents[i]==null)) {
				if(respondents[i].getPoints()!=(-1)){
				results[respondents[i].getPoints()] += ((double)1/respondents.length)*100;
				System.out.println("Alt " + respondents[i].getPoints() + " " + ((double)1/respondents.length)*100 + "%" );
				//System.out.println(respondents[i].getName() + " points " + respondents[i].getPoints());
				}
			}
		}
		barList.removeAll(barList);
		correctAnswer = question.getAlternatives().get((question.getCorrectAlternativeIndex()));
		for(int i=0;i<question.getAlternatives().size();i++){
			Color color = colorList.get(i);
			barList.add(new BarGraphics((float)color.getRed()/255,(float)color.getGreen()/255,(float)color.getBlue()/255, results[i]));
		}
		GL gl = glDrawable.getGL();
		gl.glDisable(GL.GL_TEXTURE_2D);
		done = false;
		correctZ = 0;
	}
	
	public boolean displayFeedback(GLAutoDrawable glDrawable, boolean isLastQuestion){
		if((!done)&&(!correctAnswer.equals(""))){
			final GL gl = glDrawable.getGL();
			gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
			gl.glLoadIdentity();
			textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
			if(isLastQuestion){
				//if((!(roundRespondents==null))&&(allBarsStoppedAssumer)){
				if((allBarsStoppedAssumer)){
					textRenderer.begin3DRendering();
					textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
					textRenderer.draw3D("Round results:", -9.0f,6.2f, -18-0f, 0.016f);
					if(!(roundRespondents==null)){
					for(int i=0;i<3;i++){
						if(roundRespondents.length>i){
						textRenderer.draw3D(i+1 + ". " + roundRespondents[i].getName() + " " + roundRespondents[i].getPoints() + " p ", (float) (-8.0f + (i*3.5)+3),6.2f, -18-0f, 0.016f);
						}
					}
					}
					textRenderer.end3DRendering();
				}
			}
			int altSize = question.getAlternatives().size();
			//gl.glTranslatef(-4.0f, -3.0f, question.getAlternatives().size()*-3.5f);
			gl.glTranslatef((float)-altSize -1,  -2.0f ,(float) ((float)-12.5f - (altSize*1.2)));
			gl.glRotatef(20.0f-rtri, 20.0f, 20.0f, 0.0f);
			BarGraphics bar = null;
			textRenderer.begin3DRendering();
			textRenderer.setColor(0.4f, 0.4f, 0.4f, 1.0f);
			textRenderer.draw3D("100%",-2.1f , 4.9f,1.0f, 0.01f);
			textRenderer.draw3D("50%",-2.1f , 2.4f,1.0f, 0.01f);
			textRenderer.end3DRendering();
	        gl.glBegin(GL.GL_QUADS);    
	        gl.glColor3f(0.2f, 0.2f, 0.2f);
	        gl.glVertex3f(-1.2f, 5.0f, 0.9f);			
	        gl.glVertex3f(12.0f, 5.0f, 0.9f);	
	        gl.glVertex3f(12.0f, 4.95f, 0.9f);	
	        gl.glVertex3f(-1.2f, 4.95f, 0.9f);			
	        //gl.glVertex3f(10.0f, 0.0f, -0.9f);			
	        
	        gl.glVertex3f(-1.2f, 2.5f, 0.9f);			
	        gl.glVertex3f(12.0f, 2.5f, 0.9f);	
	        gl.glVertex3f(12.0f, 2.45f, 0.9f);	
	        gl.glVertex3f(-1.2f, 2.45f, 0.9f);			
	        //gl.glVertex3f(10.0f, 0.0f, -0.9f);		
	        
	        gl.glEnd();	
	        allBarsStoppedAssumer  = true;
			for(int i=0;i<barList.size();i++){
				bar = barList.get(i);
				bar.renderBar(glDrawable);
	
				
				textRenderer.begin3DRendering();
				textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
				String tempAlt = question.getAlternatives().get(i);
				String tempAlt2 = "";
				String tempAlt1 = "";
				if(tempAlt.length()>12){
					tempAlt1 = tempAlt.substring(0,13) + "-";
					if(tempAlt.length()>26){
						tempAlt2 = tempAlt.substring(12,25);
					} else {
						tempAlt2 = tempAlt.substring(13,tempAlt.length());
					}
				} else {
					tempAlt1 = tempAlt;
				}
				if(barList.get(i).isRevealAnswer()){
					textRenderer.draw3D(tempAlt1, -1.0f, -1.3f+(correctZ/4), correctZ, 0.016f);
					textRenderer.draw3D(tempAlt2, -1.0f, -2.0f+(correctZ/4), correctZ, 0.016f);
					if(correctZ<2.6)
					correctZ += 0.07f;
				} else {
					textRenderer.draw3D(tempAlt1, -1.0f, -1.5f, 0.0f, 0.01f);
					textRenderer.draw3D(tempAlt2, -1.0f, -2.0f, 0.0f, 0.01f);
				}
				textRenderer.draw3D(bar.getDrawnScore()+ " %", 0.5f, bar.getCurrentHeigth()+0.05f, 1.0f, 0.01f);
				gl.glTranslatef(3.0f, 0.0f, 0.0f);
				if(!bar.isBarStopped()){
					allBarsStoppedAssumer = false;
				}
				textRenderer.end3DRendering();

			}
			if(allBarsStoppedAssumer){
				if(!soundPlayed ){
					barList.get(question.getCorrectAlternativeIndex()).playSound();
					soundPlayed = true;
				}
				barList.get(question.getCorrectAlternativeIndex()).setRevealAnswer(true);
				textRenderer.begin3DRendering();
				
				textRenderer.setColor(1.0f, 1.0f, 1.0f, barList.get(question.getCorrectAlternativeIndex()).getFeedBackAlpha());
				textRenderer.draw3D(barList.get(question.getCorrectAlternativeIndex()).getFeedback(), -3.0f*(barList.size()-question.getCorrectAlternativeIndex()), barList.get(question.getCorrectAlternativeIndex()).getFlyHeigth(), 0.9f , 0.012f);
				textRenderer.end3DRendering();
				//if(barList.get(question.getCorrectAlternativeIndex()).getFeedBackAlpha()<0.001f){
				if(continueRequested){
					done = true;
					bar.reset();
					rtri = 0.0f;
					continueRequested = false;
					return false;
				}
				gl.glLoadIdentity();
				textRenderer.begin3DRendering();
				gl.glColor3f(1.0f, 1.0f, 1.0f);
				//textRenderer.draw3D("Correct: " + question.getAlternatives().get(question.getCorrectAlternativeIndex()), -02.3f, 2.0f, -7.0f, 0.007f);
				textRenderer.end3DRendering();
			} else {
            rtri += 0.1f;
			}
			gl.glFlush();
		} 	  
		return true;
	}

	public void resetFeedBack(GLAutoDrawable glDrawable){
		glDrawable.getGL().glEnable(GL.GL_TEXTURE_2D);
	}
	
	public void setRoundRespondents(Respondent[] respondents) {
		Arrays.sort(respondents);
		roundRespondents = respondents;
		
	}

	public void nextStep() {
		continueRequested  = true;
		
	}
	
	
}
