package graphics;

import com.sun.opengl.util.j2d.TextRenderer;
import com.sun.opengl.util.texture.Texture;
import com.sun.org.apache.bcel.internal.generic.GETSTATIC;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;
import java.util.Arrays;
import sun.java2d.pipe.CompositePipe;


import java.applet.AudioClip;
import java.awt.Font;
import java.beans.FeatureDescriptor;
import java.io.IOException;
import java.nio.IntBuffer;
import java.text.NumberFormat;

/**
 * The renderer class controls the flow of the rendering thread across methods and objects 
 * according to the selected game mode. The graphics that are not sepcific to game mode are implemented 
 * in the methods of this class, as well as sound effects and the general logic and state info for the 
 * flow of the graphical rendering.
 * 
 * 
 * @author Terje
 *
 */
public class Renderer implements GLEventListener {
	private GLU glu = new GLU();
	private NumberFormat numberFormat;
	private TextRenderer textRenderer;
	private TextRenderer hiResTextRenderer;
	private TextRenderer midResTextRenderer;
	private QuestionGraphics tempQuestion = null;
	
	private GraphicsFacade facade;
	private int texture;
	private int countdowntexture;
	private boolean countdownRendered = false;
	private float rot;
	private long startTime = 0;
	private float clockRot = 0;
	private long timemark = 0;
	private int timeLimit = 0;
	private boolean questionInitialized = false;
	private boolean roundInitialized = false;
	private AudioSample secondSound;
	private AudioSample timesUpSound;
	private boolean isLastQuestion = false;
	
	boolean displayFirst = false;
	boolean displaySec = false;
	boolean displayThird = false;
	int printAverage = 0;
	
	TextureReader.Texture idleTexture = null;
	TextureReader.Texture countdownTexture = null;

	private GameMode currentGameMode;
	private boolean freshResults = false;
	private Respondent[] tempRespondendts = null;
	private Respondent[] tempSumRespondents = null;
	private boolean isLastRound = false;
	private Respondent[] tempSumGameRespondents = null;
	private double gameAverage = 0;
	private AudioSample gameOverSound;
	private boolean gameOverSoundPlayed = false;

	/**
	 * 
	 * initialize variables, sound clips.
	 * 
	 * @param facade
	 */
	public Renderer(GraphicsFacade facade) {
		rot = 0.0f;
		this.facade = facade;
		numberFormat = NumberFormat.getNumberInstance();
		numberFormat.setMinimumFractionDigits(2);
		numberFormat.setMaximumFractionDigits(2);
		textRenderer = new TextRenderer(new Font("Trebuchet MS", Font.PLAIN, 24));
		hiResTextRenderer = new TextRenderer(new Font("Trebuchet MS", Font.BOLD, 85));
		midResTextRenderer = new TextRenderer(new Font("Trebuchet MS", Font.BOLD, 50));
		
		try {
			secondSound = new  AudioSample(ResourceRetriever.getResourceAsStream("click.wav"));
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			timesUpSound = new  AudioSample(ResourceRetriever.getResourceAsStream("clock.wav"));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		try {
			gameOverSound = new  AudioSample(ResourceRetriever.getResourceAsStream("smas-smb3_game_over.wav"));
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public void initializeLoginIdle(GLAutoDrawable glDrawable){
		isLastQuestion = false;
		isLastRound = false;
		facade.getQuestionGraphics().setQuestion("");
		if(!(currentGameMode==null)){
		currentGameMode.resetFeedBack(glDrawable);
		}
		GL gl = glDrawable.getGL();
		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
		countdownRendered = false;
		System.out.println("IDLE INITIALIZED");
		makeRGBTexture(gl, glu, idleTexture, GL.GL_TEXTURE_2D, false);

	}

	/*
	 * Render to screen a logo and the welcome messages as people log in.
	 */
	void renderLoginIdle(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL(); 
		gl.glLoadIdentity();	
		gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
		// Reset The Current Modelview Matrix
		gl.glTranslatef(0.0f, 0.0f, -15.0f);						// Move One Unit Into The Screen
		//System.out.println(facade.getLoginQueue().size());
		if(facade.getLoginQueue().size()>0){
			textRenderer.begin3DRendering();
			textRenderer.setColor(1.0f, 1.0f, 1.0f, facade.getLoginQueue().get(0).alpha);
			textRenderer.draw3D(facade.getLoginQueue().get(0).text + " logged in",facade.getLoginQueue().get(0).x , facade.getLoginQueue().get(0).y,Constants.FALLING_TEXT_Z_COORD, 0.030f);
			textRenderer.end3DRendering();
			//textRenderer.draw3D(facade.getLoginQueue().get(0).text, facade.getLoginQueue().get(0).x, facade.getLoginQueue().get(0).y, Constants.FALLING_TEXT_Z_COORD, 1.0f);
			facade.getLoginQueue().get(0).updateState();
			if(facade.getLoginQueue().get(0).alpha<=0.0f)
				facade.getLoginQueue().remove(0);

		}
		if(!facade.getLectureCode().equals("")){
			textRenderer.begin3DRendering();
			textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
			textRenderer.draw3D("Today's lecture code : " + facade.getLectureCode(),-6.0f , -7.0f,Constants.FALLING_TEXT_Z_COORD, 0.030f);
			textRenderer.end3DRendering();

		}
		// gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();
		gl.glColor3f(1.0f, 1.0f, 1.0f);


		gl.glTranslatef(-0.25f, 0.0f, -6.0f);
		gl.glBegin(GL.GL_QUADS);  

		gl.glTexCoord2f(0.0f, 0.0f);
		gl.glVertex3f(-1.0f, -1.0f, 1.0f);
		gl.glTexCoord2f(1.0f, 0.0f);

		gl.glVertex3f(1.0f, -1.0f, 1.0f);
		gl.glTexCoord2f(1.0f, 1.0f);
		gl.glVertex3f(1.0f, 1.0f, 1.0f);
		gl.glTexCoord2f(0.0f, 1.0f);
		gl.glVertex3f(-1.0f, 1.0f, 1.0f);

		gl.glEnd();				// Done Drawing The Quad
		gl.glFlush();
		rot+=1.0f;

	}
	
	/**
	 * Render the high score list to screen
	 * 
	 * @param glDrawable
	 * @return
	 */
	public boolean renderEndGame(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
		gl.glLoadIdentity();
		gl.glDisable(GL.GL_TEXTURE_2D);
		textRenderer.begin3DRendering();

		textRenderer.draw3D("Today's winners", -8.0f, 6.0f, -18.0f, 0.017f);
		float offset = 0;
		if(!(tempSumGameRespondents==null)){
				if(((tempSumGameRespondents.length>0)&&(displayFirst||(System.currentTimeMillis()%4000)>3500))){
					textRenderer.draw3D("1." + tempSumGameRespondents[0].getName() + " : " + tempSumGameRespondents[0].getPoints() + " points " , 
							-8.0f, 4.0f, -18.0f, 0.022f);
					displayFirst = true;
				}
				if(((tempSumGameRespondents.length>1)&&(displayFirst||(System.currentTimeMillis()%4000)>3000))){
					textRenderer.draw3D("2." + tempSumGameRespondents[1].getName() + " : " + tempSumGameRespondents[1].getPoints() + " points " , 
							-8.0f, 2.0f, -18.0f, 0.022f);
					displaySec = true;
				}
				if(((tempSumGameRespondents.length>2)&&(displayFirst||(System.currentTimeMillis()%4000)>2500))){
					textRenderer.draw3D("3." + tempSumGameRespondents[2].getName() + " : " + tempSumGameRespondents[2].getPoints() + " points " , 
							-8.0f, 0.0f, -18.0f, 0.022f);
					displayThird = true;
				}
		}

		if(new Double(gameAverage).toString().length()>4){
			textRenderer.draw3D("Average score : " + new Double(gameAverage).toString().substring(0,4) + " points", -8.0f, -5.0f, -18.0f, 0.022f);
		} else {
			textRenderer.draw3D("Average score : " + gameAverage + " points", -8.0f, -5.0f, -18.0f, 0.022f);	
		}
			textRenderer.end3DRendering();
		
		gl.glFlush();



		//average score
		return true;
	}
	
	/**
	 * Render the posing of the question, with alternatives and the animated timer.
	 * 
	 * @param glDrawable
	 */
	public void renderPopQuestion(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		if((!questionInitialized)&&(!facade.getQuestionGraphics().getQuestion().equals(""))){
			timeLimit = facade.getQuestionGraphics().getTimeLimit();
			clockRot = 0;
			startTime = 0;
			questionInitialized = true;
		}
		if(!countdownRendered){
			
			makeRGBTexture(gl, glu, countdownTexture, GL.GL_TEXTURE_2D, false);
			countdownRendered=true;
		}

		gl.glLoadIdentity();
		gl.glColor3f(1.0f, 1.0f, 1.0f);


		gl.glTranslatef(-0.25f, 3.5f, -15.0f);

		if(questionInitialized&&(timeLimit>0)){
			double tempInterval;
			if(startTime!=0){
				tempInterval = System.currentTimeMillis() - startTime;
				timemark += tempInterval;
				clockRot += (float) (tempInterval*0.36);
				gl.glRotatef(clockRot, 0.0f, 0.0f, 1.0f);
				if(timemark>1000){
					if(timeLimit<5 && timeLimit>1){
						secondSound.play();
					} else if (timeLimit==1){
						timesUpSound.play();
					}
					timeLimit-=1;
					
					timemark = 0;
				}   	 
			} 
			startTime = System.currentTimeMillis();
		} else if(questionInitialized){
			if((System.currentTimeMillis() - startTime)>2500)
				if(freshResults)
					nextStep();
		}


		gl.glBegin(GL.GL_QUADS);  

		gl.glTexCoord2f(0.0f, 0.0f);
		gl.glVertex3f(-1.0f, -1.0f, 1.0f);
		gl.glTexCoord2f(1.0f, 0.0f);

		gl.glVertex3f(1.0f, -1.0f, 1.0f);
		gl.glTexCoord2f(1.0f, 1.0f);
		gl.glVertex3f(1.0f, 1.0f, 1.0f);
		gl.glTexCoord2f(0.0f, 1.0f);
		gl.glVertex3f(-1.0f, 1.0f, 1.0f);

		gl.glEnd();				// Done Drawing The Quad
		gl.glFlush();
		rot+=1.0f;

		gl.glLoadIdentity();									// Reset The Current Modelview Matrix
		gl.glTranslatef(0.0f, 0.0f, -30.0f);						// Move One Unit Into The Screen       	 	

		if(!facade.getQuestionGraphics().getQuestion().equals("")){
			hiResTextRenderer.begin3DRendering();
			hiResTextRenderer.setColor(0.0f, 0.0f, 0.0f, 0.9f);
			//String temp = "10";
			String temp = new Integer(timeLimit).toString();
			if(temp.length()<2)
				temp = " " + temp;
			hiResTextRenderer.draw3D(temp, -1.1f, 3.0f,16.0f, 0.015f);
			hiResTextRenderer.end3DRendering();
			gl.glTranslatef(0.0f, 4.0f, 0.0f);
			textRenderer.begin3DRendering();
			textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
			String tempQuestion = facade.getQuestionGraphics().getQuestion();
			int offset = 0;
			while(tempQuestion.length()>60){
			 
			 int cutIndex = tempQuestion.substring(0,60).lastIndexOf(" ");
			 textRenderer.draw3D(tempQuestion.substring(0,cutIndex),-18.5f ,( -1.5f*offset),Constants.FALLING_TEXT_Z_COORD, 0.05f);
			 tempQuestion = tempQuestion.substring(cutIndex+1,tempQuestion.length());
			 offset++;
			}

			 textRenderer.draw3D(tempQuestion,-18.5f , -1.5f*offset,Constants.FALLING_TEXT_Z_COORD, 0.05f);
			textRenderer.setColor(1.0f, 1.0f, 1.0f, 1.0f);
			for(int i=0;i<facade.getQuestionGraphics().getAlternatives().size();i++){
				textRenderer.draw3D(i+1 +". " + facade.getQuestionGraphics().getAlternatives().get(i),-15.5f ,((float)(-1.5*(i+offset))-(2)),Constants.FALLING_TEXT_Z_COORD, 0.05f);
			}
			textRenderer.end3DRendering();
			//textRenderer.draw3D(facade.getLoginQueue().get(0).text, facade.getLoginQueue().get(0).x, facade.getLoginQueue().get(0).y, Constants.FALLING_TEXT_Z_COORD, 1.0f);
		} 
	}




/**
 * OpenGL specific einitializtion.
 */
	public void init(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		gl.glShadeModel(GL.GL_SMOOTH);							// Enable Smooth Shading
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
		gl.glClearDepth(1.0f);									// Depth Buffer Setup

		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glEnable(GL.GL_BLEND);// Enables Depth Testing
		gl.glDepthFunc(GL.GL_LEQUAL);								// The Type Of Depth Testing To Do
		gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
		gl.glEnable(GL.GL_LIGHT0);								// Enable Default Light (Quick And Dirty)
		gl.glEnable(GL.GL_LIGHTING);								// Enable Lighting
		gl.glEnable(GL.GL_COLOR_MATERIAL);						// Enable Coloring Of Material
		gl.glEnable(GL.GL_TEXTURE_2D);
		texture = genTexture(gl);
		countdowntexture = genTexture(gl);
		gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
		
		try {
			countdownTexture = TextureReader.readTexture("timerback.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		
		
		try {
			idleTexture = TextureReader.readTexture("lecturegame.bmp");
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		makeRGBTexture(gl, glu, idleTexture, GL.GL_TEXTURE_2D, false);
		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);

	}

	
	/**
	 * This method is called for each frame to be displayed. The current state of the program decides which 
	 * method performs the actual rendering.
	 */
	public synchronized void display(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
		if(facade.getScreenState()==Constants.INIT_IDLE_LOGIN_STATE){
			initializeLoginIdle(glDrawable);
			facade.setScreenState(Constants.IDLE_LOGIN_STATE);
		} else if(facade.getScreenState()==Constants.RENDER_TOTAL_FEEDBACK){
			if(!gameOverSoundPlayed ){
				gameOverSound.play();
				gameOverSoundPlayed = true;
			}
			renderEndGame(glDrawable);
		}
		else if(facade.getScreenState()==Constants.IDLE_LOGIN_STATE){
			renderLoginIdle(glDrawable);
		} else if(facade.getScreenState()==Constants.INITIALIZE_FEEDBACK){
			if(tempRespondendts==null){
				//if no respondents just skip the feedback
				freshResults = false;
				questionInitialized = false;
				facade.setScreenState(Constants.INIT_IDLE_LOGIN_STATE);
				
			}
			else if(freshResults){
				currentGameMode.initFeedback(tempQuestion,tempRespondendts,glDrawable);
				freshResults = false;
				questionInitialized = false;
				facade.setScreenState(Constants.RENDER_FEEDBACK);
			} else {
					renderPopQuestion(glDrawable);
			}
		}else if(facade.getScreenState()==Constants.POP_QUESTION){
				renderPopQuestion(glDrawable);
		}else if(facade.getScreenState()==Constants.RENDER_ROUND_INFO){
			if(!currentGameMode.displayRoundIntro(glDrawable)){
				facade.netSend("@^QST^@");
				facade.setScreenState(Constants.POP_QUESTION);
				
			}
		} else if(facade.getScreenState()==Constants.RENDER_FEEDBACK){
			if(!currentGameMode.displayFeedback(glDrawable,isLastQuestion)) {
				if((isLastRound)&&(isLastQuestion)){
					facade.setScreenState(Constants.RENDER_TOTAL_FEEDBACK);
				} else {
					facade.setScreenState(Constants.INIT_IDLE_LOGIN_STATE);
				}
			}
		}
	}

	/**
	 * OpenGL specific reshaping. See nehe.gamedev.net for documentation.
	 * 
	 */
	public void reshape(GLAutoDrawable glDrawable, int x, int y, int w, int h) {
		if (h == 0) h = 1;
		GL gl = glDrawable.getGL();
		gl.glViewport(0, 0, w, h);                       // Reset The Current Viewport And Perspective Transformation
		gl.glMatrixMode(GL.GL_PROJECTION);                           // Select The Projection Matrix
		gl.glLoadIdentity();                                      // Reset The Projection Matrix
		glu.gluPerspective(45.0f, (float) w / (float) h, 0.1f, 100.0f);  // Calculate The Aspect Ratio Of The Window
		gl.glMatrixMode(GL.GL_MODELVIEW);                            // Select The Modelview Matrix
		gl.glLoadIdentity();                                      // Reset The ModalView Matrix
	}

	public void displayChanged(GLAutoDrawable glDrawable, boolean b, boolean b1) {
	}

	/**
	 * 
	 * Jumps to the next logical step as the user requests next step or the previous step 
	 * was finished.
	 *
	 */
	public void nextStep(){
		if(facade.getScreenState()==Constants.RENDER_FEEDBACK){
			currentGameMode.nextStep();
			return;
		}
		System.out.println("nextStep kalt. Screenstate = " + facade.getScreenState());
		if(facade.getScreenState()==Constants.IDLE_LOGIN_STATE) {
			if(roundInitialized){
			facade.setScreenState(Constants.POP_QUESTION);
			facade.netSend("@^QST^@");
			} else {
				roundInitialized = true;
				while(currentGameMode==null){
					try {
						Thread.sleep(300);
					} catch (InterruptedException e) {

						e.printStackTrace();
					}
				}
				currentGameMode.initRoundIntro();
				facade.setScreenState(Constants.RENDER_ROUND_INFO);
			}
		}

		else if(facade.getScreenState()==Constants.POP_QUESTION){
			facade.setScreenState(Constants.INITIALIZE_FEEDBACK);
		}   
		else if(facade.getScreenState()==Constants.SHOW_RESULTS){
			facade.setScreenState(Constants.INIT_IDLE_LOGIN_STATE);
		}
	}

	/**
	 * 
	 * OpenGL helper methid for preaparation of textures.
	 * 
	 * @param gl
	 * @param glu
	 * @param img
	 * @param target
	 * @param mipmapped
	 */
	private void makeRGBTexture(GL gl, GLU glu, TextureReader.Texture img, int target, boolean mipmapped) {
		if (mipmapped) {

			glu.gluBuild2DMipmaps(target, GL.GL_RGB, img.getWidth(), img.getHeight(), GL.GL_RGB, GL.GL_UNSIGNED_BYTE, img.getPixels());
		} else {
			gl.glTexImage2D(target, 0, GL.GL_RGB, img.getWidth(), img.getHeight(), 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE, img.getPixels());
		}
	}
/**
 * OpenGL helper methid for preaparation of textures.
 * 
 * @param gl
 * @return
 */
	private int genTexture(GL gl) {
		final int[] tmp = new int[1];
		gl.glGenTextures(1, tmp, 0);
		return tmp[0];
	}

/**
 * 
 * A call to this method sets the current game mode to the selectid integer. The integer must be the same on the 
 * server for meaningful processing.
 * 
 * @param mode
 */
	public void setMode(int mode) {
		while((facade.getScreenState()!=Constants.IDLE_LOGIN_STATE)){
			try {
				Thread.sleep(300);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		if(mode==Constants.MODE_BEAT_THE_SCORE){
			currentGameMode = new MeasureUpMode();
		}
		else if(mode==Constants.MODE_LAST_MAN_STANDING){
			currentGameMode = new EliminationMode();
		}
		currentGameMode.initMode(textRenderer);
		roundInitialized = false;

	}

	public boolean isQuestionInitialized() {
		return questionInitialized;
	}



	public void setQuestionInitialized(boolean questionInitialized) {
		this.questionInitialized = questionInitialized;
	}



	public synchronized void returnResults(QuestionGraphics question,Respondent[] respondents) {
		tempQuestion = question;
		tempRespondendts = respondents;
		freshResults = true;
		
	}

	public void setLastQuestion(boolean isLastQuestion) {
		System.out.println("lastQuestion:" + isLastQuestion);
		this.isLastQuestion = isLastQuestion;
	}

	public void setLastRound(boolean b) {
		System.out.println("lastround:" + b);
		isLastRound  = b;
		
	}

	public void returnRoundResults(Respondent[] respondents) {
		Arrays.sort(respondents);
		tempSumRespondents = respondents;
		currentGameMode.setRoundRespondents(respondents);

	}
	
	public void returnGameResults(Respondent[] respondents) {
		isLastRound = true;
		isLastQuestion = true;
		Arrays.sort(respondents);
		int sum = 0;
		for(int i=0; i<respondents.length;i++){
				sum += respondents[i].getPoints();
		}
		gameAverage = (int)(double)(sum/respondents.length);
		
		tempSumGameRespondents  = respondents;

	}
}
